@@ -15,27 +15,32 @@
 
 import webob
 
+from nova.api.openstack import api_version_request
 from nova.api.openstack import common
-from nova.api.openstack.compute.schemas import create_backup
-from nova.api.openstack import extensions
+from nova.api.openstack.compute.schemas import create_backup as schema
 from nova.api.openstack import wsgi
 from nova.api import validation
-from nova import compute
+from nova.compute import api as compute
 from nova import exception
 from nova.policies import create_backup as cb_policies
 
-ALIAS = "os-create-backup"
-
 
 class CreateBackupController(wsgi.Controller):
-    def __init__(self, *args, **kwargs):
-        super(CreateBackupController, self).__init__(*args, **kwargs)
+    def __init__(self):
+        super(CreateBackupController, self).__init__()
         self.compute_api = compute.API()
 
-    @extensions.expected_errors((400, 403, 404, 409))
+    @wsgi.response(202)
+    @wsgi.expected_errors((400, 403, 404, 409))
     @wsgi.action('createBackup')
-    @validation.schema(create_backup.create_backup_v20, '2.0', '2.0')
-    @validation.schema(create_backup.create_backup, '2.1')
+    @validation.schema(schema.create_backup_v20, '2.0', '2.0')
+    @validation.schema(schema.create_backup, '2.1')
+    @validation.response_body_schema(
+        schema.create_backup_response, '2.1', '2.44',
+    )
+    @validation.response_body_schema(
+        schema.create_backup_response_v245, '2.45'
+    )
     def _create_backup(self, req, id, body):
         """Backup a server instance.
 
@@ -48,7 +53,9 @@ class CreateBackupController(wsgi.Controller):
 
         """
         context = req.environ["nova.context"]
-        context.can(cb_policies.BASE_POLICY_NAME)
+        instance = common.get_instance(self.compute_api, context, id)
+        context.can(cb_policies.BASE_POLICY_NAME,
+                    target={'project_id': instance.project_id})
         entity = body["createBackup"]
 
         image_name = common.normalize_name(entity["name"])
@@ -57,22 +64,27 @@ class CreateBackupController(wsgi.Controller):
 
         props = {}
         metadata = entity.get('metadata', {})
-        common.check_img_metadata_properties_quota(context, metadata)
+        # Starting from microversion 2.39 we don't check quotas on createBackup
+        if api_version_request.is_supported(
+                req, max_version=
+                api_version_request.MAX_IMAGE_META_PROXY_API_VERSION):
+            common.check_img_metadata_properties_quota(context, metadata)
         props.update(metadata)
 
-        instance = common.get_instance(self.compute_api, context, id)
-
         try:
             image = self.compute_api.backup(context, instance, image_name,
                     backup_type, rotation, extra_properties=props)
-        except exception.InstanceUnknownCell as e:
-            raise webob.exc.HTTPNotFound(explanation=e.format_message())
         except exception.InstanceInvalidState as state_error:
             common.raise_http_conflict_for_instance_invalid_state(state_error,
                     'createBackup', id)
         except exception.InvalidRequest as e:
             raise webob.exc.HTTPBadRequest(explanation=e.format_message())
 
+        # Starting with microversion 2.45 we return a response body containing
+        # the snapshot image id without the Location header.
+        if api_version_request.is_supported(req, '2.45'):
+            return {'image_id': image['id']}
+
         resp = webob.Response(status_int=202)
 
         # build location of newly-created image entity if rotation is not zero
@@ -83,19 +95,3 @@ class CreateBackupController(wsgi.Controller):
             resp.headers['Location'] = image_ref
 
         return resp
-
-
-class CreateBackup(extensions.V21APIExtensionBase):
-    """Create a backup of a server."""
-
-    name = "CreateBackup"
-    alias = ALIAS
-    version = 1
-
-    def get_controller_extensions(self):
-        controller = CreateBackupController()
-        extension = extensions.ControllerExtension(self, 'servers', controller)
-        return [extension]
-
-    def get_resources(self):
-        return []
