@@ -15,7 +15,6 @@
 #    under the License.
 
 import functools
-import six
 
 import nova.api.openstack
 from nova.api.openstack.compute import admin_actions
@@ -38,7 +37,6 @@ from nova.api.openstack.compute import evacuate
 from nova.api.openstack.compute import extension_info
 from nova.api.openstack.compute import fixed_ips
 from nova.api.openstack.compute import flavor_access
-from nova.api.openstack.compute import flavor_manage
 from nova.api.openstack.compute import flavors
 from nova.api.openstack.compute import flavors_extraspecs
 from nova.api.openstack.compute import floating_ip_dns
@@ -74,6 +72,7 @@ from nova.api.openstack.compute import server_groups
 from nova.api.openstack.compute import server_metadata
 from nova.api.openstack.compute import server_migrations
 from nova.api.openstack.compute import server_password
+from nova.api.openstack.compute import server_shares
 from nova.api.openstack.compute import server_tags
 from nova.api.openstack.compute import server_topology
 from nova.api.openstack.compute import servers
@@ -96,7 +95,7 @@ def _create_controller(main_controller, action_controller_list):
 
     controller = wsgi.Resource(main_controller())
     for ctl in action_controller_list:
-        controller.register_actions(ctl())
+        controller.register_subcontroller_actions(ctl())
     return controller
 
 
@@ -144,7 +143,6 @@ fixed_ips_controller = functools.partial(_create_controller,
 flavor_controller = functools.partial(_create_controller,
     flavors.FlavorsController,
     [
-        flavor_manage.FlavorManageController,
         flavor_access.FlavorActionController
     ]
 )
@@ -313,6 +311,8 @@ server_remote_consoles_controller = functools.partial(_create_controller,
 server_security_groups_controller = functools.partial(_create_controller,
     security_groups.ServerSecurityGroupController, [])
 
+server_shares_controller = functools.partial(_create_controller,
+    server_shares.ServerSharesController, [])
 
 server_tags_controller = functools.partial(_create_controller,
     server_tags.ServerTagsController, [])
@@ -828,6 +828,14 @@ ROUTE_LIST = (
     ('/servers/{server_id}/os-security-groups', {
         'GET': [server_security_groups_controller, 'index']
     }),
+    ('/servers/{server_id}/shares', {
+        'GET': [server_shares_controller, 'index'],
+        'POST': [server_shares_controller, 'create'],
+    }),
+    ('/servers/{server_id}/shares/{id}', {
+        'GET': [server_shares_controller, 'show'],
+        'DELETE': [server_shares_controller, 'delete'],
+    }),
     ('/servers/{server_id}/tags', {
         'GET': [server_tags_controller, 'index'],
         'PUT': [server_tags_controller, 'update_all'],
@@ -849,6 +857,7 @@ class APIRouterV21(base_wsgi.Router):
     and method. The URL mapping based on the plain list `ROUTE_LIST` is built
     at here.
     """
+
     def __init__(self, custom_routes=None):
         """:param custom_routes: the additional routes can be added by this
                parameter. This parameter is used to test on some fake routes
@@ -866,7 +875,7 @@ class APIRouterV21(base_wsgi.Router):
             # For example, the request to the '' will be redirect to the '/' in
             # the Nova API. To indicate that, using the target path instead of
             # a dict. The route entry just writes as "('', '/)".
-            if isinstance(methods, six.string_types):
+            if isinstance(methods, str):
                 self.map.redirect(path, methods)
                 continue
 
@@ -880,5 +889,8 @@ class APIRouterV21(base_wsgi.Router):
 
     @classmethod
     def factory(cls, global_config, **local_config):
-        """Simple paste factory, :class:`nova.wsgi.Router` doesn't have one."""
+        """Simple paste factory.
+
+        :class:`nova.api.wsgi.Router` doesn't have one.
+        """
         return cls()
