@@ -30,3 +30,27 @@ rescue = {
     'required': ['rescue'],
     'additionalProperties': False,
 }
+
+# TODO(stephenfin): Restrict the value to 'null' in a future API version
+unrescue = {
+    'type': 'object',
+    'properties': {
+        'unrescue': {},
+    },
+    'required': ['unrescue'],
+    'additionalProperties': False,
+}
+
+rescue_response = {
+    'type': 'object',
+    'properties': {
+        'adminPass': {
+            'type': 'string',
+        },
+    },
+    'additionalProperties': False,
+}
+
+unrescue_response = {
+    'type': 'null',
+}
