@@ -14,6 +14,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
+
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 
@@ -39,7 +41,7 @@ from nova import utils
 LOG = logging.getLogger(__name__)
 
 
-AZ_NOT_IN_REQUEST_SPEC = object()
+SCHED_HINTS_NOT_IN_REQUEST_SPEC = object()
 
 
 class ViewBuilder(common.ViewBuilder):
@@ -97,7 +99,8 @@ class ViewBuilder(common.ViewBuilder):
     def basic(self, request, instance, show_extra_specs=False,
               show_extended_attr=None, show_host_status=None,
               show_sec_grp=None, bdms=None, cell_down_support=False,
-              show_user_data=False, provided_az=None):
+              show_user_data=False, provided_az=None,
+              provided_sched_hints=None):
         """Generic, non-detailed view of an instance."""
         if cell_down_support and 'display_name' not in instance:
             # NOTE(tssurya): If the microversion is >= 2.69, this boolean will
@@ -222,29 +225,41 @@ class ViewBuilder(common.ViewBuilder):
         return unknown_only
 
     def _get_pinned_az(self, context, instance, provided_az):
-        if provided_az is AZ_NOT_IN_REQUEST_SPEC:
-            # Case the provided_az is pre fetched, but not specified
-            pinned_az = None
-        elif provided_az is not None:
-            # Case the provided_az is pre fetched, and specified
+        pinned_az = ''
+        if provided_az is not None:
             pinned_az = provided_az
         else:
-            # Case the provided_az is not pre fethed.
             try:
                 req_spec = objects.RequestSpec.get_by_instance_uuid(
                         context, instance.uuid)
                 pinned_az = req_spec.availability_zone
             except exception.RequestSpecNotFound:
-                pinned_az = None
+                pinned_az = ''
         return pinned_az
 
+    def _get_scheduler_hints(self, context, instance, provided_sched_hints):
+        if provided_sched_hints is SCHED_HINTS_NOT_IN_REQUEST_SPEC:
+            # Case where it was pre fetched, but not specified
+            sched_hints = None
+        elif provided_sched_hints is not None:
+            sched_hints = provided_sched_hints
+        else:
+            # Case the provided_az is not pre fethed.
+            try:
+                req_spec = objects.RequestSpec.get_by_instance_uuid(
+                        context, instance.uuid)
+                sched_hints = req_spec.scheduler_hints
+            except exception.RequestSpecNotFound:
+                sched_hints = {}
+        return sched_hints
+
     def show(self, request, instance, extend_address=True,
              show_extra_specs=None, show_AZ=True, show_config_drive=True,
              show_extended_attr=None, show_host_status=None,
              show_keypair=True, show_srv_usg=True, show_sec_grp=True,
              show_extended_status=True, show_extended_volumes=True,
              bdms=None, cell_down_support=False, show_server_groups=False,
-             show_user_data=True, provided_az=None):
+             show_user_data=True, provided_az=None, provided_sched_hints=None):
         """Detailed view of a single instance."""
         if show_extra_specs is None:
             # detail will pre-calculate this for us. If we're doing show,
@@ -305,6 +320,7 @@ class ViewBuilder(common.ViewBuilder):
             server["server"]["progress"] = instance.get("progress", 0)
 
         context = request.environ['nova.context']
+
         if show_AZ:
             az = avail_zone.get_instance_availability_zone(context, instance)
             # NOTE(mriedem): The OS-EXT-AZ prefix should not be used for new
@@ -315,6 +331,11 @@ class ViewBuilder(common.ViewBuilder):
                 pinned_az = self._get_pinned_az(context, instance, provided_az)
                 server['server']['pinned_availability_zone'] = pinned_az
 
+        if api_version_request.is_supported(request, min_version='2.100'):
+            server['server']['scheduler_hints'] = (
+                    self._get_scheduler_hints(
+                        context, instance, provided_sched_hints))
+
         if show_config_drive:
             server["server"]["config_drive"] = instance["config_drive"]
 
@@ -522,28 +543,36 @@ class ViewBuilder(common.ViewBuilder):
         :returns: Server data in dictionary format
         """
         req_specs = None
-        req_specs_dict = {}
+        req_specs_dict = collections.defaultdict(str)
+        sched_hints_dict = {}
         if api_version_request.is_supported(request, min_version='2.96'):
             context = request.environ['nova.context']
             instance_uuids = [s.uuid for s in servers]
             req_specs = objects.RequestSpec.get_by_instance_uuids(
                     context, instance_uuids)
-            req_specs_dict.update({req.instance_uuid: req.availability_zone
-                                   for req in req_specs
-                                   if req.availability_zone is not None})
+            req_specs_dict = {req.instance_uuid: req.availability_zone
+                              for req in req_specs}
+            if api_version_request.is_supported(request, min_version='2.100'):
+                sched_hints_dict.update({
+                    req.instance_uuid: req.scheduler_hints
+                    for req in req_specs
+                    if req.scheduler_hints is not None})
 
-        server_list = [func(request, server,
-                            show_extra_specs=show_extra_specs,
-                            show_extended_attr=show_extended_attr,
-                            show_host_status=show_host_status,
-                            show_sec_grp=show_sec_grp, bdms=bdms,
-                            cell_down_support=cell_down_support,
-                            provided_az=req_specs_dict.get(
-                                server.uuid, AZ_NOT_IN_REQUEST_SPEC))["server"]
-                       for server in servers
-                       # Filter out the fake marker instance created by the
-                       # fill_virtual_interface_list online data migration.
-                       if server.uuid != virtual_interface.FAKE_UUID]
+        server_list = [
+            func(request, server,
+                 show_extra_specs=show_extra_specs,
+                 show_extended_attr=show_extended_attr,
+                 show_host_status=show_host_status,
+                 show_sec_grp=show_sec_grp, bdms=bdms,
+                 cell_down_support=cell_down_support,
+                 provided_az=req_specs_dict[server.uuid],
+                 provided_sched_hints=sched_hints_dict.get(
+                     server.uuid, SCHED_HINTS_NOT_IN_REQUEST_SPEC)
+                 )["server"]
+            for server in servers
+            # Filter out the fake marker instance created by the
+            # fill_virtual_interface_list online data migration.
+            if server.uuid != virtual_interface.FAKE_UUID]
         servers_links = self._get_collection_links(request,
                                                    servers,
                                                    coll_name)
@@ -591,13 +620,26 @@ class ViewBuilder(common.ViewBuilder):
             bookmark = self._image_builder._get_bookmark_link(request,
                                                               image_id,
                                                               "images")
-            return {
+            image = {
                 "id": image_id,
                 "links": [{
                     "rel": "bookmark",
                     "href": bookmark,
                 }],
             }
+
+            if api_version_request.is_supported(request, min_version='2.98'):
+                image_props = {}
+                for key, value in instance.system_metadata.items():
+                    if key.startswith(utils.SM_IMAGE_PROP_PREFIX):
+                        # remove prefix 'image_' at start of key, so that
+                        # key 'image_<key-name>' becomes '<key-name>'
+                        k = key.partition('_')[2]
+                        image_props[k] = value
+
+                image['properties'] = image_props
+
+            return image
         else:
             return ""
 
