@@ -12,9 +12,12 @@
 """WSGI application initialization for Nova APIs."""
 
 import os
+import sys
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_reports import guru_meditation_report as gmr
+from oslo_reports import opts as gmr_opts
 from oslo_service import _options as service_opts
 from paste import deploy
 
@@ -24,6 +27,7 @@ from nova import exception
 from nova import objects
 from nova import service
 from nova import utils
+from nova import version
 
 CONF = cfg.CONF
 
@@ -38,15 +42,21 @@ def _get_config_files(env=None):
     if env is None:
         env = os.environ
     dirname = env.get('OS_NOVA_CONFIG_DIR', '/etc/nova').strip()
+    files = env.get('OS_NOVA_CONFIG_FILES', '').split(';')
+    if files == ['']:
+        files = CONFIG_FILES
     return [os.path.join(dirname, config_file)
-            for config_file in CONFIG_FILES]
+            for config_file in files]
 
 
 def _setup_service(host, name):
     try:
         utils.raise_if_old_compute()
     except exception.TooOldComputeService as e:
-        logging.getLogger(__name__).warning(str(e))
+        if CONF.workarounds.disable_compute_service_check_for_ffu:
+            LOG.warning(str(e))
+        else:
+            raise
 
     binary = name if name.startswith('nova-') else "nova-%s" % name
 
@@ -81,13 +91,19 @@ def error_application(exc, name):
 
 @utils.run_once('Global data already initialized, not re-initializing.',
                 LOG.info)
-def init_global_data(conf_files):
+def init_global_data(conf_files, service_name):
     # NOTE(melwitt): parse_args initializes logging and calls global rpc.init()
     # and db_api.configure(). The db_api.configure() call does not initiate any
     # connection to the database.
-    config.parse_args([], default_config_files=conf_files)
+
+    # NOTE(gibi): sys.argv is set by the wsgi runner e.g. uwsgi sets it based
+    # on the --pyargv parameter of the uwsgi binary
+    config.parse_args(sys.argv, default_config_files=conf_files)
 
     logging.setup(CONF, "nova")
+    gmr_opts.set_defaults(CONF)
+    gmr.TextGuruMeditation.setup_autorun(
+        version, conf=CONF, service_name=service_name)
 
     # dump conf at debug (log_options option comes from oslo.service)
     # FIXME(mriedem): This is gross but we don't have a public hook into
@@ -109,7 +125,7 @@ def init_application(name):
     # apache/mod_wsgi reloads the init_application script. So, we initialize
     # global data separately and decorate the method to run only once in a
     # python interpreter instance.
-    init_global_data(conf_files)
+    init_global_data(conf_files, name)
 
     try:
         _setup_service(CONF.host, name)
