@@ -403,6 +403,9 @@ class ComputeAPI(object):
         * ...  - Rename the instance_type argument of resize_instance() to
                  flavor
         * 6.1 - Add reimage_boot_volume parameter to rebuild_instance()
+        * 6.2 - Add target_state parameter to rebuild_instance()
+        * 6.3 - Add delete_attachment parameter to remove_volume_connection
+        * 6.4 - Add allow_share() and deny_share()
     '''
 
     VERSION_ALIASES = {
@@ -424,6 +427,11 @@ class ComputeAPI(object):
         'xena': '6.0',
         'yoga': '6.0',
         'zed': '6.1',
+        'antelope': '6.2',
+        'bobcat': '6.2',
+        'caracal': '6.3',
+        'dalmatian': '6.3',
+        'epoxy': '6.4',
     }
 
     @property
@@ -834,13 +842,6 @@ class ComputeAPI(object):
         return cctxt.call(ctxt, 'get_spice_console',
                           instance=instance, console_type=console_type)
 
-    def get_rdp_console(self, ctxt, instance, console_type):
-        version = self._ver(ctxt, '5.0')
-        cctxt = self.router.client(ctxt).prepare(
-                server=_compute_host(None, instance), version=version)
-        return cctxt.call(ctxt, 'get_rdp_console',
-                          instance=instance, console_type=console_type)
-
     def get_mks_console(self, ctxt, instance, console_type):
         version = self._ver(ctxt, '5.0')
         cctxt = self.router.client(ctxt).prepare(
@@ -1083,7 +1084,7 @@ class ComputeAPI(object):
             image_ref, orig_image_ref, orig_sys_metadata, bdms,
             recreate, on_shared_storage, host, node,
             preserve_ephemeral, migration, limits, request_spec, accel_uuids,
-            reimage_boot_volume):
+            reimage_boot_volume, target_state):
 
         # NOTE(edleafe): compute nodes can only use the dict form of limits.
         if isinstance(limits, objects.SchedulerLimits):
@@ -1096,11 +1097,19 @@ class ComputeAPI(object):
             'limits': limits,
             'request_spec': request_spec,
             'accel_uuids': accel_uuids,
-            'reimage_boot_volume': reimage_boot_volume
+            'reimage_boot_volume': reimage_boot_volume,
+            'target_state': target_state,
         }
-
-        version = '6.1'
+        version = '6.2'
         client = self.router.client(ctxt)
+        if not client.can_send_version(version):
+            if msg_args['target_state']:
+                raise exception.UnsupportedRPCVersion(
+                    api="rebuild_instance",
+                    required="6.2")
+            else:
+                del msg_args['target_state']
+            version = '6.1'
         if not client.can_send_version(version):
             if msg_args['reimage_boot_volume']:
                 raise exception.NovaException(
@@ -1129,12 +1138,23 @@ class ComputeAPI(object):
         cctxt.cast(ctxt, 'remove_fixed_ip_from_instance',
                    instance=instance, address=address)
 
-    def remove_volume_connection(self, ctxt, instance, volume_id, host):
-        version = self._ver(ctxt, '5.0')
-        cctxt = self.router.client(ctxt).prepare(
-                server=host, version=version)
-        return cctxt.call(ctxt, 'remove_volume_connection',
-                          instance=instance, volume_id=volume_id)
+    def remove_volume_connection(
+                self, ctxt, instance, volume_id, host,
+                delete_attachment=False
+            ):
+        version = '6.3'
+        client = self.router.client(ctxt)
+        kwargs = {
+            'instance': instance,
+            'volume_id': volume_id,
+            'delete_attachment': delete_attachment
+        }
+        if not client.can_send_version(version):
+            kwargs.pop('delete_attachment')
+            version = self._ver(ctxt, '5.0')
+
+        cctxt = client.prepare(server=host, version=version)
+        return cctxt.call(ctxt, 'remove_volume_connection', **kwargs)
 
     def rescue_instance(self, ctxt, instance, rescue_password,
                         rescue_image_ref=None, clean_shutdown=True):
@@ -1459,6 +1479,38 @@ class ComputeAPI(object):
                    volume_id=volume_id, snapshot_id=snapshot_id,
                    delete_info=delete_info)
 
+    def allow_share(self, ctxt, instance, share_mapping):
+        version = '6.4'
+        client = self.router.client(ctxt)
+        if not client.can_send_version(version):
+            raise exception.UnsupportedRPCVersion(
+                    api="allow_share",
+                    required=version)
+        cctxt = self.router.client(ctxt).prepare(
+                server=_compute_host(None, instance), version=version)
+        cctxt.cast(
+            ctxt,
+            "allow_share",
+            instance=instance,
+            share_mapping=share_mapping
+        )
+
+    def deny_share(self, ctxt, instance, share_mapping):
+        version = '6.4'
+        client = self.router.client(ctxt)
+        if not client.can_send_version(version):
+            raise exception.UnsupportedRPCVersion(
+                    api="deny_share",
+                    required=version)
+        cctxt = self.router.client(ctxt).prepare(
+                server=_compute_host(None, instance), version=version)
+        cctxt.cast(
+            ctxt,
+            "deny_share",
+            instance=instance,
+            share_mapping=share_mapping,
+        )
+
     def external_instance_event(self, ctxt, instances, events, host=None):
         instance = instances[0]
         version = self._ver(ctxt, '5.0')
