@@ -39,19 +39,6 @@ its middleware, NoAuthMiddleware[V2_18], will be removed in a future release.
 """,
         help="""
 Determine the strategy to use for authentication.
-"""),
-    cfg.BoolOpt("use_forwarded_for",
-        default=False,
-        deprecated_for_removal=True,
-        deprecated_reason='This feature is duplicate of the HTTPProxyToWSGI '
-                          'middleware in oslo.middleware',
-        deprecated_group="DEFAULT",
-        deprecated_since='26.0.0',
-        help="""
-When True, the 'X-Forwarded-For' header is treated as the canonical remote
-address. When False (the default), the 'remote_address' header is used.
-
-You should only enable this if you have an HTML sanitizing proxy.
 """),
 ]
 
@@ -225,8 +212,11 @@ service.
         help="""
 Domain name used to configure FQDN for instances.
 
-Configure a fully-qualified domain name for instance hostnames. If unset, only
-the hostname without a domain will be configured.
+Configure a fully-qualified domain name for instance hostnames. The value is
+suffixed to the instance hostname from the database to construct the hostname
+that appears in the metadata API. To disable this behavior (for example in
+order to correctly support microversion's 2.94 FQDN hostnames), set this to the
+empty string.
 
 Possible values:
 
@@ -406,12 +396,45 @@ so if your hypervisor does not support password injection, set this to False.
 """)
 ]
 
+validation_opts = [
+    cfg.StrOpt(
+        "response_validation",
+        choices=(
+            (
+                "error",
+                "Raise a HTTP 500 (Server Error) for responses that fail "
+                "response body schema validation",
+            ),
+            (
+                "warn",
+                "Log a warning for responses that fail response body schema "
+                "validation",
+            ),
+            (
+                "ignore",
+                "Ignore response body schema validation failures",
+            ),
+        ),
+        default="warn",
+        help="""\
+Configure validation of API responses.
+
+``warn`` is the current recommendation for production environments. This is
+expected to change to ``error`` in a future release.
+
+If you find it necessary to enable the ``ignore`` option, please report the
+issues you are seeing to the Nova team so we can improve our schemas.
+""",
+    ),
+]
+
 API_OPTS = (auth_opts +
             metadata_opts +
             file_opts +
             osapi_opts +
             os_network_opts +
-            enable_inst_pw_opts)
+            enable_inst_pw_opts +
+            validation_opts)
 
 
 def register_opts(conf):
