@@ -161,8 +161,8 @@ class Service(BASE, NovaBase, models.SoftDeleteMixin):
     version = sa.Column(sa.Integer, default=0)
 
     instance = orm.relationship(
-        "Instance",
-        backref='services',
+        'Instance',
+        back_populates='services',
         primaryjoin='and_(Service.host == Instance.host,'
                     'Service.binary == "nova-compute",'
                     'Instance.deleted == 0)',
@@ -266,7 +266,6 @@ class Instance(BASE, NovaBase, models.SoftDeleteMixin):
     """Represents a guest VM."""
     __tablename__ = 'instances'
     __table_args__ = (
-        sa.Index('uuid', 'uuid', unique=True),
         sa.Index('instances_project_id_idx', 'project_id'),
         sa.Index('instances_project_id_deleted_idx',
               'project_id', 'deleted'),
@@ -286,6 +285,8 @@ class Instance(BASE, NovaBase, models.SoftDeleteMixin):
               'deleted', 'created_at'),
         sa.Index('instances_updated_at_project_id_idx',
               'updated_at', 'project_id'),
+        sa.Index('instances_compute_id_deleted_idx',
+              'compute_id', 'deleted'),
         schema.UniqueConstraint('uuid', name='uniq_instances0uuid'),
     )
     injected_files = []
@@ -346,6 +347,10 @@ class Instance(BASE, NovaBase, models.SoftDeleteMixin):
     # To identify the "ComputeNode" which the instance resides in.
     # This equals to ComputeNode.hypervisor_hostname.
     node = sa.Column(sa.String(255))
+    # This identifies the ComputeNode object that this instance resides
+    # on and should be equivalent to the one referenced by the 'node'
+    # field above.
+    compute_id = sa.Column(sa.BigInteger())
 
     # *not* flavorid, this is the internal primary_key
     instance_type_id = sa.Column(sa.Integer)
@@ -420,6 +425,96 @@ class Instance(BASE, NovaBase, models.SoftDeleteMixin):
 
     hidden = sa.Column(sa.Boolean, default=False)
 
+    block_device_mapping = orm.relationship(
+        'BlockDeviceMapping',
+        back_populates='instance',
+        primaryjoin=(
+            'and_(BlockDeviceMapping.instance_uuid == Instance.uuid, '
+            'BlockDeviceMapping.deleted == 0)'
+        ),
+    )
+    console_auth_tokens = orm.relationship(
+        'ConsoleAuthToken',
+        back_populates='instance',
+        foreign_keys='ConsoleAuthToken.instance_uuid',
+        primaryjoin=(
+            'and_(Instance.uuid == ConsoleAuthToken.instance_uuid,'
+            'Instance.deleted == 0)'
+        ),
+    )
+    extra = orm.relationship(
+        'InstanceExtra',
+        back_populates='instance',
+        uselist=False,
+    )
+    info_cache = orm.relationship(
+        'InstanceInfoCache',
+        back_populates='instance',
+        uselist=False,
+    )
+    pci_devices = orm.relationship(
+        'PciDevice',
+        back_populates='instance',
+        foreign_keys='PciDevice.instance_uuid',
+        primaryjoin=(
+            'and_(Instance.uuid == PciDevice.instance_uuid,'
+            'PciDevice.deleted == 0)'
+        ),
+    )
+    services = orm.relationship(
+        'Service',
+        back_populates='instance',
+        primaryjoin=(
+            'and_(Instance.host == Service.host,'
+            'Service.binary == "nova-compute",'
+            'Instance.deleted == 0)'
+        ),
+        foreign_keys='Service.host',
+    )
+    security_groups = orm.relationship(
+        'SecurityGroup',
+        secondary='security_group_instance_association',
+        back_populates='instances',
+        primaryjoin=(
+            'and_('
+            'SecurityGroupInstanceAssociation.instance_uuid == Instance.uuid,'
+            # (anthony) the condition below shouldn't be necessary now that the
+            # association is being marked as deleted.  However, removing this
+            # may cause existing deployments to choke, so I'm leaving it
+            'Instance.deleted == 0)'
+        ),
+        secondaryjoin=(
+            'and_('
+            'SecurityGroup.id == SecurityGroupInstanceAssociation.security_group_id,'  # noqa: E501
+            'SecurityGroupInstanceAssociation.deleted == 0,'
+            'SecurityGroup.deleted == 0)'
+        ),
+    )
+    system_metadata = orm.relationship(
+        'InstanceSystemMetadata',
+        back_populates='instance',
+    )
+    tags = orm.relationship(
+        'Tag',
+        back_populates='instance',
+        primaryjoin=(
+            'and_(Instance.uuid == Tag.resource_id,Instance.deleted == 0)'
+        ),
+        foreign_keys='Tag.resource_id',
+    )
+
+
+# NOTE(stephenfin): https://github.com/sqlalchemy/sqlalchemy/discussions/8619
+Instance.metadata = orm.relationship(
+    'InstanceMetadata',
+    back_populates='instance',
+    foreign_keys='InstanceMetadata.instance_uuid',
+    primaryjoin=(
+        'and_(Instance.uuid == InstanceMetadata.instance_uuid,'
+        'InstanceMetadata.deleted == 0)'
+    ),
+)
+
 
 class InstanceInfoCache(BASE, NovaBase, models.SoftDeleteMixin):
     """Represents a cache of information about an instance
@@ -437,7 +532,7 @@ class InstanceInfoCache(BASE, NovaBase, models.SoftDeleteMixin):
     instance_uuid = sa.Column(sa.String(36), sa.ForeignKey('instances.uuid'),
                            nullable=False)
     instance = orm.relationship(Instance,
-                            backref=orm.backref('info_cache', uselist=False),
+                            back_populates='info_cache',
                             foreign_keys=instance_uuid,
                             primaryjoin=instance_uuid == Instance.uuid)
 
@@ -461,8 +556,7 @@ class InstanceExtra(BASE, NovaBase, models.SoftDeleteMixin):
     # and can be removed in the future release.
     resources = orm.deferred(sa.Column(sa.Text))
     instance = orm.relationship(Instance,
-                            backref=orm.backref('extra',
-                                                uselist=False),
+                            back_populates='extra',
                             foreign_keys=instance_uuid,
                             primaryjoin=instance_uuid == Instance.uuid)
 
@@ -611,7 +705,7 @@ class BlockDeviceMapping(BASE, NovaBase, models.SoftDeleteMixin):
     # transition period first.
     uuid = sa.Column(sa.String(36))
     instance = orm.relationship(Instance,
-                            backref=orm.backref('block_device_mapping'),
+                            back_populates='block_device_mapping',
                             foreign_keys=instance_uuid,
                             primaryjoin='and_(BlockDeviceMapping.'
                                               'instance_uuid=='
@@ -659,10 +753,53 @@ class BlockDeviceMapping(BASE, NovaBase, models.SoftDeleteMixin):
     encryption_format = sa.Column(sa.String(128))
     encryption_options = sa.Column(sa.String(4096))
 
+
+class ShareMapping(BASE, NovaBase):
+    """Represents share / instance mapping."""
+    __tablename__ = "share_mapping"
+    __table_args__ = (
+        sa.Index('share_idx', 'share_id'),
+        sa.Index('share_mapping_instance_uuid_share_id_idx',
+              'instance_uuid', 'share_id'),
+        sa.UniqueConstraint(
+            "instance_uuid",
+            "share_id",
+            name="uniq_key_pairs0instance_uuid0share_id",
+        ),
+        sa.UniqueConstraint(
+            "instance_uuid",
+            "tag",
+            name="uniq_key_pairs0instance_uuid0tag",
+        ),
+    )
+    # sqlite> create table my_table(id bigint primary key AUTOINCREMENT,
+    # name text);
+    # Parse error: AUTOINCREMENT is only allowed on an INTEGER PRIMARY KEY
+    # Use BigInteger variant for sqlite to allow unit tests. Other database
+    # should support BigInteger and autoincrement.
+    id = sa.Column(
+        sa.BigInteger().with_variant(sa.Integer, "sqlite"),
+        primary_key=True,
+        autoincrement=True,
+        nullable=False,
+    )
+    uuid = sa.Column(sa.String(36))
+    instance_uuid = sa.Column(sa.String(36), sa.ForeignKey('instances.uuid'))
+    instance = orm.relationship(
+        "Instance",
+        foreign_keys=instance_uuid,
+        primaryjoin='and_(ShareMapping.instance_uuid == Instance.uuid,'
+                    'Instance.deleted == 0)'
+    )
+    share_id = sa.Column(sa.String(36))
+    status = sa.Column(sa.String(32))
+    tag = sa.Column(sa.String(48))
+    export_location = sa.Column(sa.Text)
+    share_proto = sa.Column(sa.String(32))
+
+
 # TODO(stephenfin): Remove once we drop the security_groups field from the
 # Instance table. Until then, this is tied to the SecurityGroup table
-
-
 class SecurityGroupInstanceAssociation(BASE, NovaBase, models.SoftDeleteMixin):
     __tablename__ = 'security_group_instance_association'
     __table_args__ = (
@@ -693,6 +830,7 @@ class SecurityGroup(BASE, NovaBase, models.SoftDeleteMixin):
     project_id = sa.Column(sa.String(255))
 
     instances = orm.relationship(Instance,
+                             back_populates='security_groups',
                              secondary = "security_group_instance_association",
                              primaryjoin = 'and_('
         'SecurityGroup.id == '
@@ -704,8 +842,17 @@ class SecurityGroup(BASE, NovaBase, models.SoftDeleteMixin):
         # (anthony) the condition below shouldn't be necessary now that the
         # association is being marked as deleted.  However, removing this
         # may cause existing deployments to choke, so I'm leaving it
-        'Instance.deleted == 0)',
-                             backref='security_groups')
+        'Instance.deleted == 0)')
+
+    rules = orm.relationship(
+        'SecurityGroupIngressRule',
+        primaryjoin=(
+            'and_('
+            'SecurityGroupIngressRule.parent_group_id == SecurityGroup.id,'
+            'SecurityGroupIngressRule.deleted == 0)'
+        ),
+        back_populates='parent_group',
+    )
 
 
 # TODO(stephenfin): Remove once we drop the security_groups field from the
@@ -718,8 +865,9 @@ class SecurityGroupIngressRule(BASE, NovaBase, models.SoftDeleteMixin):
 
     parent_group_id = sa.Column(
         sa.Integer, sa.ForeignKey('security_groups.id'))
-    parent_group = orm.relationship("SecurityGroup", backref="rules",
-                                foreign_keys=parent_group_id,
+    parent_group = orm.relationship("SecurityGroup",
+                                back_populates="rules",
+                                foreign_keys=[parent_group_id],
                                 primaryjoin='and_('
         'SecurityGroupIngressRule.parent_group_id == SecurityGroup.id,'
         'SecurityGroupIngressRule.deleted == 0)')
@@ -733,7 +881,7 @@ class SecurityGroupIngressRule(BASE, NovaBase, models.SoftDeleteMixin):
     # granting access for.
     group_id = sa.Column(sa.Integer, sa.ForeignKey('security_groups.id'))
     grantee_group = orm.relationship("SecurityGroup",
-                                 foreign_keys=group_id,
+                                 foreign_keys=[group_id],
                                  primaryjoin='and_('
         'SecurityGroupIngressRule.group_id == SecurityGroup.id,'
         'SecurityGroupIngressRule.deleted == 0)')
@@ -750,6 +898,8 @@ class Migration(BASE, NovaBase, models.SoftDeleteMixin):
               'status'),
         sa.Index('migrations_uuid', 'uuid', unique=True),
         sa.Index('migrations_updated_at_idx', 'updated_at'),
+        sa.Index('migrations_dest_compute_id_deleted_idx',
+                 'dest_compute_id', 'deleted'),
     )
     id = sa.Column(sa.Integer, primary_key=True, nullable=False)
     # NOTE(tr3buchet): the ____compute variables are instance['host']
@@ -758,6 +908,8 @@ class Migration(BASE, NovaBase, models.SoftDeleteMixin):
     # nodes are equivalent to a compute node's 'hypervisor_hostname'
     source_node = sa.Column(sa.String(255))
     dest_node = sa.Column(sa.String(255))
+    # The ID of the ComputeNode that matches dest_node
+    dest_compute_id = sa.Column(sa.BigInteger())
     # NOTE(tr3buchet): dest_host, btw, is an ip address
     dest_host = sa.Column(sa.String(255))
     old_instance_type_id = sa.Column(sa.Integer())
@@ -815,12 +967,16 @@ class InstanceMetadata(BASE, NovaBase, models.SoftDeleteMixin):
     key = sa.Column(sa.String(255))
     value = sa.Column(sa.String(255))
     instance_uuid = sa.Column(sa.String(36), sa.ForeignKey('instances.uuid'))
-    instance = orm.relationship(Instance, backref="metadata",
-                            foreign_keys=instance_uuid,
-                            primaryjoin='and_('
-                                'InstanceMetadata.instance_uuid == '
-                                     'Instance.uuid,'
-                                'InstanceMetadata.deleted == 0)')
+
+    instance = orm.relationship(
+        Instance,
+        back_populates="metadata",
+        foreign_keys=instance_uuid,
+        primaryjoin=(
+            'and_(InstanceMetadata.instance_uuid == Instance.uuid,'
+            'InstanceMetadata.deleted == 0)'
+        ),
+    )
 
 
 class InstanceSystemMetadata(BASE, NovaBase, models.SoftDeleteMixin):
@@ -836,8 +992,11 @@ class InstanceSystemMetadata(BASE, NovaBase, models.SoftDeleteMixin):
                            sa.ForeignKey('instances.uuid'),
                            nullable=False)
 
-    instance = orm.relationship(Instance, backref="system_metadata",
-                            foreign_keys=instance_uuid)
+    instance = orm.relationship(
+        Instance,
+        back_populates='system_metadata',
+        foreign_keys=instance_uuid,
+    )
 
 
 class VolumeUsage(BASE, NovaBase, models.SoftDeleteMixin):
@@ -1013,7 +1172,8 @@ class PciDevice(BASE, NovaBase, models.SoftDeleteMixin):
     numa_node = sa.Column(sa.Integer, nullable=True)
 
     parent_addr = sa.Column(sa.String(12), nullable=True)
-    instance = orm.relationship(Instance, backref="pci_devices",
+    instance = orm.relationship(Instance,
+                                back_populates="pci_devices",
                                 foreign_keys=instance_uuid,
                                 primaryjoin='and_('
                                 'PciDevice.instance_uuid == Instance.uuid,'
@@ -1031,11 +1191,11 @@ class Tag(BASE, models.ModelBase):
     tag = sa.Column(sa.Unicode(80), primary_key=True, nullable=False)
 
     instance = orm.relationship(
-        "Instance",
-        backref='tags',
+        'Instance',
+        back_populates='tags',
+        foreign_keys=resource_id,
         primaryjoin='and_(Tag.resource_id == Instance.uuid,'
                     'Instance.deleted == 0)',
-        foreign_keys=resource_id
     )
 
 
@@ -1046,7 +1206,6 @@ class ConsoleAuthToken(BASE, NovaBase):
     __table_args__ = (
         sa.Index('console_auth_tokens_instance_uuid_idx', 'instance_uuid'),
         sa.Index('console_auth_tokens_host_expires_idx', 'host', 'expires'),
-        sa.Index('console_auth_tokens_token_hash_idx', 'token_hash'),
         sa.Index(
             'console_auth_tokens_token_hash_instance_uuid_idx', 'token_hash',
             'instance_uuid',
@@ -1059,6 +1218,7 @@ class ConsoleAuthToken(BASE, NovaBase):
     console_type = sa.Column(sa.String(255), nullable=False)
     host = sa.Column(sa.String(255), nullable=False)
     port = sa.Column(sa.Integer, nullable=False)
+    tls_port = sa.Column(sa.Integer, nullable=True)
     internal_access_path = sa.Column(sa.String(255))
     instance_uuid = sa.Column(sa.String(36), nullable=False)
     expires = sa.Column(sa.Integer, nullable=False)
@@ -1066,7 +1226,7 @@ class ConsoleAuthToken(BASE, NovaBase):
 
     instance = orm.relationship(
         "Instance",
-        backref='console_auth_tokens',
+        back_populates='console_auth_tokens',
         primaryjoin='and_(ConsoleAuthToken.instance_uuid == Instance.uuid,'
                     'Instance.deleted == 0)',
         foreign_keys=instance_uuid
