@@ -34,7 +34,6 @@ from nova import exception
 from nova.i18n import _
 from nova.network import neutron as neutronapi
 from nova.objects import security_group as security_group_obj
-from nova import utils
 
 
 LOG = logging.getLogger(__name__)
@@ -204,18 +203,6 @@ def _rule_exists(security_group, new_rule):
     return False
 
 
-def validate_property(value, property, allowed):
-    """Validate given security group property.
-
-    :param value:    the value to validate, as a string or unicode
-    :param property: the property, either 'name' or 'description'
-    :param allowed:  the range of characters allowed, but not used because
-                     Neutron is allowing any characters.
-    """
-    utils.check_string_length(value, name=property, min_length=0,
-                              max_length=255)
-
-
 def populate_security_groups(security_groups):
     """Build and return a SecurityGroupList.
 
@@ -649,6 +636,8 @@ def add_to_instance(context, instance, security_group_name):
         except n_exc.NeutronClientException as e:
             if e.status_code == 400:
                 raise exception.SecurityGroupCannotBeApplied(str(e))
+            elif e.status_code == 409:
+                raise exception.SecurityGroupConnectionStateConflict(str(e))
             else:
                 raise e
         except Exception:
