@@ -130,10 +130,10 @@ class ImageMetaPropsPayload(base.NotificationPayloadBase):
     #                     'hw_ephemeral_encryption_format' fields
     # Version 1.11: Added 'hw_locked_memory' field
     # Version 1.12: Added 'hw_viommu_model' field
-    VERSION = '1.12'
-
-    SCHEMA = {
-        k: ('image_meta_props', k) for k in image_meta.ImageMetaProps.fields}
+    # Version 1.13: Added 'hw_virtio_packed_ring' field
+    # Version 1.14: Added 'hw_firmware_stateless' field
+    # Version 1.15: Added igb value to 'hw_vif_model' enum
+    VERSION = '1.15'
 
     # NOTE(efried): This logic currently relies on all of the fields of
     # ImageMetaProps being initialized with no arguments. See the docstring.
@@ -142,7 +142,11 @@ class ImageMetaPropsPayload(base.NotificationPayloadBase):
     #  But it is not clear that OVO can tolerate the same *instance* of a type
     #  class being used in more than one place.
     fields = {
-        k: v.__class__() for k, v in image_meta.ImageMetaProps.fields.items()}
+        k: v.__class__() for k, v in image_meta.ImageMetaProps.fields.items()
+            if k not in ('hw_ephemeral_encryption_secret_uuid',)}
+
+    SCHEMA = {
+        k: ('image_meta_props', k) for k in fields}
 
     def __init__(self, image_meta_props):
         super(ImageMetaPropsPayload, self).__init__()
