@@ -125,7 +125,7 @@ def _metadata_add_to_db(context, aggregate_id, metadata, max_retries=10,
                                         "aggregate_id": aggregate_id})
                 if new_entries:
                     context.session.execute(
-                        api_models.AggregateMetadata.__table__.insert(None),
+                        api_models.AggregateMetadata.__table__.insert(),
                         new_entries)
 
                 return metadata
@@ -437,7 +437,7 @@ def _get_by_host_from_db(context, host, key=None):
 
 @api_db_api.context_manager.reader
 def _get_by_metadata_from_db(context, key=None, value=None):
-    assert(key is not None or value is not None)
+    assert key is not None or value is not None
     query = context.session.query(api_models.Aggregate)
     query = query.join(api_models.Aggregate._metadata)
     if key is not None:
