@@ -526,6 +526,27 @@ class RNGModel(BaseNovaEnum):
     ALL = (VIRTIO,)
 
 
+class ShareMappingStatus(BaseNovaEnum):
+    """Represents the possible status of a share mapping"""
+
+    ATTACHING = "attaching"
+    DETACHING = "detaching"
+    ACTIVE = "active"
+    INACTIVE = "inactive"
+    ERROR = "error"
+
+    ALL = (ATTACHING, DETACHING, ACTIVE, INACTIVE, ERROR)
+
+
+class ShareMappingProto(BaseNovaEnum):
+    """Represents the possible protocol used by a share mapping"""
+
+    NFS = "NFS"
+    CEPHFS = "CEPHFS"
+
+    ALL = (NFS, CEPHFS)
+
+
 class TPMModel(BaseNovaEnum):
 
     TIS = "tpm-tis"
@@ -541,6 +562,13 @@ class TPMVersion(BaseNovaEnum):
     ALL = (v1_2, v2_0)
 
 
+class MaxPhyAddrMode(BaseNovaEnum):
+    PASSTHROUGH = "passthrough"
+    EMULATE = "emulate"
+
+    ALL = (PASSTHROUGH, EMULATE)
+
+
 class SCSIModel(BaseNovaEnum):
 
     BUSLOGIC = "buslogic"
@@ -808,8 +836,8 @@ class DiskFormat(BaseNovaEnum):
     ALL = (RBD, LVM, QCOW2, RAW, PLOOP, VHD, VMDK, VDI, ISO)
 
 
-# TODO(stephenfin): Remove the xenapi value when we bump the 'Diagnostics'
-# object (the only user of this enum) to 2.0
+# TODO(stephenfin): Remove the xenapi and hyperv value when we bump the
+# 'Diagnostics' object (the only user of this enum) to 2.0
 class HypervisorDriver(BaseNovaEnum):
     LIBVIRT = "libvirt"
     XENAPI = "xenapi"
@@ -909,6 +937,8 @@ class NotificationAction(BaseNovaEnum):
     RESCUE = 'rescue'
     VOLUME_ATTACH = 'volume_attach'
     VOLUME_DETACH = 'volume_detach'
+    SHARE_ATTACH = 'share_attach'
+    SHARE_DETACH = 'share_detach'
     CREATE = 'create'
     IMPORT = 'import'
     EVACUATE = 'evacuate'
@@ -949,9 +979,9 @@ class NotificationAction(BaseNovaEnum):
     ALL = (UPDATE, EXCEPTION, DELETE, PAUSE, UNPAUSE, RESIZE, VOLUME_SWAP,
            SUSPEND, POWER_ON, REBOOT, SHUTDOWN, SNAPSHOT, INTERFACE_ATTACH,
            POWER_OFF, SHELVE, RESUME, RESTORE, EXISTS, RESCUE, VOLUME_ATTACH,
-           VOLUME_DETACH, CREATE, IMPORT, EVACUATE, RESIZE_FINISH,
-           LIVE_MIGRATION_ABORT, LIVE_MIGRATION_POST_DEST, LIVE_MIGRATION_POST,
-           LIVE_MIGRATION_PRE, LIVE_MIGRATION_ROLLBACK,
+           VOLUME_DETACH, SHARE_ATTACH, SHARE_DETACH, CREATE, IMPORT, EVACUATE,
+           RESIZE_FINISH, LIVE_MIGRATION_ABORT, LIVE_MIGRATION_POST_DEST,
+           LIVE_MIGRATION_POST, LIVE_MIGRATION_PRE, LIVE_MIGRATION_ROLLBACK,
            LIVE_MIGRATION_ROLLBACK_DEST, REBUILD, INTERFACE_DETACH,
            RESIZE_CONFIRM, RESIZE_PREP, RESIZE_REVERT, SHELVE_OFFLOAD,
            SOFT_DELETE, TRIGGER_CRASH_DUMP, UNRESCUE, UNSHELVE, ADD_HOST,
@@ -1275,6 +1305,10 @@ class InputBusField(BaseEnumField):
     AUTO_TYPE = InputBus()
 
 
+class MaxPhysAddrModeField(BaseEnumField):
+    AUTO_TYPE = MaxPhyAddrMode()
+
+
 class MigrationTypeField(BaseEnumField):
     AUTO_TYPE = MigrationType()
 
@@ -1287,6 +1321,14 @@ class RNGModelField(BaseEnumField):
     AUTO_TYPE = RNGModel()
 
 
+class ShareMappingStatusField(BaseEnumField):
+    AUTO_TYPE = ShareMappingStatus()
+
+
+class ShareMappingProtoField(BaseEnumField):
+    AUTO_TYPE = ShareMappingProto()
+
+
 class TPMModelField(BaseEnumField):
     AUTO_TYPE = TPMModel()
 
