@@ -188,6 +188,8 @@ VIR_CONNECT_LIST_DOMAINS_INACTIVE = 2
 VIR_CONNECT_LIST_NODE_DEVICES_CAP_PCI_DEV = 2
 VIR_CONNECT_LIST_NODE_DEVICES_CAP_NET = 1 << 4
 VIR_CONNECT_LIST_NODE_DEVICES_CAP_VDPA = 1 << 17
+VIR_CONNECT_LIST_NODE_DEVICES_CAP_MDEV = 1 << 5
+VIR_CONNECT_LIST_NODE_DEVICES_INACTIVE = 1 << 8
 
 # secret type
 VIR_SECRET_USAGE_TYPE_NONE = 0
@@ -517,7 +519,7 @@ class HostPCIDevicesInfo(object):
 
     def __init__(self, num_pci=0, num_pfs=2, num_vfs=8, num_mdevcap=0,
                  numa_node=None, multiple_gpu_types=False,
-                 generic_types=False):
+                 generic_types=False, bus=0x81, product_ids=["1515"]):
         """Create a new HostPCIDevicesInfo object.
 
         :param num_pci: (int) The number of (non-SR-IOV) and (non-MDEV capable)
@@ -531,6 +533,14 @@ class HostPCIDevicesInfo(object):
             split between ``$TOTAL_NUMA_NODES`` nodes.
         :param multiple_gpu_types: (bool) Supports different vGPU types
         :param generic_types: (bool) Supports both nvidia-12 and mlx5 types
+        :param bus: (int) PCI bus number, allow to specify a different bus to
+            have an asymmetric configuration between src and dst computes.
+        :param product_ids: (list) List of product IDs.
+            Warning: This multiplies the number of PF and VF devices
+            by the number of product IDs. It creates separate sets of VF
+            devices, one for each product ID.
+            The first set is assigned to slot 0, and each subsequent set
+            is assigned to the next slot incrementally.
         """
         self.devices = {}
 
@@ -543,7 +553,7 @@ class HostPCIDevicesInfo(object):
         if num_pfs and num_vfs % num_pfs:
             raise ValueError('num_vfs must be a factor of num_pfs')
 
-        bus = 0x81
+        bus = bus
         slot = 0x0
         function = 0
         iommu_group = 40  # totally arbitrary number
@@ -579,41 +589,43 @@ class HostPCIDevicesInfo(object):
         vf_ratio = num_vfs // num_pfs if num_pfs else 0
 
         # Generate PFs
-        for dev in range(num_pfs):
-            function = 0
-            numa_node_pf = self._calc_numa_node(dev, numa_node)
-
-            self.add_device(
-                dev_type='PF',
-                bus=bus,
-                slot=slot,
-                function=function,
-                iommu_group=iommu_group,
-                numa_node=numa_node_pf,
-                vf_ratio=vf_ratio)
-
-            parent = (bus, slot, function)
-            # Generate VFs
-            for _ in range(vf_ratio):
-                function += 1
-                iommu_group += 1
-
-                if function % 8 == 0:
-                    # functions must be 0-7
-                    slot += 1
-                    function = 0
+        for prod_id in product_ids:
+            for dev in range(num_pfs):
+                function = 0
+                numa_node_pf = self._calc_numa_node(dev, numa_node)
 
                 self.add_device(
-                    dev_type='VF',
+                    dev_type='PF',
                     bus=bus,
                     slot=slot,
                     function=function,
                     iommu_group=iommu_group,
                     numa_node=numa_node_pf,
-                    vf_ratio=vf_ratio,
-                    parent=parent)
+                    vf_ratio=vf_ratio)
 
-            slot += 1
+                parent = (bus, slot, function)
+                # Generate VFs
+                for _ in range(vf_ratio):
+                    function += 1
+                    iommu_group += 1
+
+                    if function % 8 == 0:
+                        # functions must be 0-7
+                        slot += 1
+                        function = 0
+
+                    self.add_device(
+                        dev_type='VF',
+                        bus=bus,
+                        prod_id=prod_id,
+                        slot=slot,
+                        function=function,
+                        iommu_group=iommu_group,
+                        numa_node=numa_node_pf,
+                        vf_ratio=vf_ratio,
+                        parent=parent)
+
+                slot += 1
 
     def add_device(
         self, dev_type, bus, slot, function, iommu_group, numa_node,
@@ -890,6 +902,13 @@ def _parse_disk_info(element):
         if not disk_info['source']:
             disk_info['source'] = source.get('path')
 
+        encryption = element.find('./source/encryption')
+        if encryption is not None and len(encryption):
+            disk_info['encryption_format'] = encryption.get('format')
+            secret = encryption.find('./secret')
+            if secret is not None:
+                disk_info['encryption_secret'] = secret.get('uuid')
+
     target = element.find('./target')
     if target is not None:
         disk_info['target_dev'] = target.get('dev')
@@ -898,6 +917,32 @@ def _parse_disk_info(element):
     return disk_info
 
 
+def _parse_vcpu_info(element):
+    vcpu_info = {}
+    vcpu_info['number'] = int(element.text)
+    vcpu_info['cpuset'] = element.get('cpuset')
+    return vcpu_info
+
+
+def _parse_filesystem_info(element):
+    filesystem_info = {}
+    filesystem_info['type'] = element.get('type', 'mount')
+
+    driver = element.find('./driver')
+    if driver is not None:
+        filesystem_info['driver_type'] = driver.get('type')
+
+    source = element.find('./source')
+    if source is not None:
+        filesystem_info['source'] = source.get('dir')
+
+    target = element.find('./target')
+    if target is not None:
+        filesystem_info['target'] = target.get('dir')
+
+    return filesystem_info
+
+
 def _parse_nic_info(element):
     nic_info = {}
     nic_info['type'] = element.get('type', 'bridge')
@@ -917,6 +962,23 @@ def _parse_nic_info(element):
     return nic_info
 
 
+def _parse_hostdev_info(element):
+    hostdev_info = {}
+    hostdev_info['type'] = element.get('type', 'pci')
+    hostdev_info['managed'] = element.get('managed', 'pci')
+
+    source = element.find('./source')
+    if source is not None:
+        address = source.find('./address')
+        if address is not None:
+            hostdev_info['domain'] = address.get('domain')
+            hostdev_info['bus'] = address.get('bus')
+            hostdev_info['slot'] = address.get('slot')
+            hostdev_info['function'] = address.get('function')
+
+    return hostdev_info
+
+
 def disable_event_thread(self):
     """Disable nova libvirt driver event thread.
 
@@ -925,7 +987,7 @@ def disable_event_thread(self):
     problematic because it means we've got a floating thread calling
     sleep(1) over the life of the unit test. Seems harmless? It's not,
     because we sometimes want to test things like retry loops that
-    should have specific sleep paterns. An unlucky firing of the
+    should have specific sleep patterns. An unlucky firing of the
     libvirt thread will cause a test failure.
 
     """
@@ -1034,7 +1096,7 @@ class NodeDevice(object):
     def attach(self):
         pass
 
-    def dettach(self):
+    def detach(self):
         pass
 
     def reset(self):
@@ -1090,7 +1152,7 @@ class Domain(object):
 
         vcpu = tree.find('./vcpu')
         if vcpu is not None:
-            definition['vcpu'] = int(vcpu.text)
+            definition['vcpu'] = _parse_vcpu_info(vcpu)
 
         memory = tree.find('./memory')
         if memory is not None:
@@ -1102,6 +1164,10 @@ class Domain(object):
             os['type'] = os_type.text
             os['arch'] = os_type.get('arch', self._connection.host_info.arch)
 
+        os_loader = tree.find('./os/loader')
+        if os_loader is not None:
+            os['loader_stateless'] = os_loader.get('stateless')
+
         os_kernel = tree.find('./os/kernel')
         if os_kernel is not None:
             os['kernel'] = os_kernel.text
@@ -1157,6 +1223,13 @@ class Domain(object):
                 disks_info += [_parse_disk_info(disk)]
             devices['disks'] = disks_info
 
+            # Manage shares
+            filesystems_info = []
+            filesystems = device_nodes.findall('./filesystem')
+            for filesystem in filesystems:
+                filesystems_info += [_parse_filesystem_info(filesystem)]
+            devices['filesystem'] = filesystems_info
+
             nics_info = []
             nics = device_nodes.findall('./interface')
             for nic in nics:
@@ -1206,6 +1279,8 @@ class Domain(object):
                         'model': hostdev.get('model'),
                         'address_uuid': address.get('uuid')
                     })
+                if dev_type == 'pci':
+                    hostdev_info.append(_parse_hostdev_info(hostdev))
             devices['hostdevs'] = hostdev_info
 
             vpmem_info = []
@@ -1338,11 +1413,12 @@ class Domain(object):
             nic_info['_attached'] = True
             self._def['devices']['nics'] += [nic_info]
             result = True
+        elif xml.startswith("<hostdev"):
+            hostdev_info = _parse_hostdev_info(etree.fromstring(xml))
+            hostdev_info['_attached'] = True
+            self._def['devices']['hostdevs'] += [hostdev_info]
+            result = True
         else:
-            # FIXME(sean-k-mooney): We don't currently handle attaching
-            # or detaching hostdevs but we have tests that assume we do so
-            # this is an error not an exception. This affects PCI passthough,
-            # vGPUs and PF neutron ports.
             LOG.error(
                 "Trying to attach an unsupported device type."
                 "The fakelibvirt implementation is incomplete "
@@ -1390,10 +1466,25 @@ class Domain(object):
 
             return attached_nic_info is not None
 
-        # FIXME(sean-k-mooney): We don't currently handle attaching or
-        # detaching hostdevs but we have tests that assume we do so this is
-        # an error not an exception. This affects PCI passthough, vGPUs and
-        # PF neutron ports
+        if xml.startswith("<hostdev"):
+            hostdev_info = _parse_hostdev_info(etree.fromstring(xml))
+            attached_hostdev_info = None
+            for attached_hostdev in self._def['devices']['hostdevs']:
+                if (
+                    attached_hostdev["domain"] == hostdev_info["domain"] and
+                    attached_hostdev["bus"] == hostdev_info["bus"] and
+                    attached_hostdev["slot"] == hostdev_info["slot"] and
+                    attached_hostdev["function"] ==
+                    hostdev_info["function"]
+                ):
+                    attached_hostdev_info = attached_hostdev
+                    break
+
+            if attached_hostdev_info:
+                self._def['devices']['hostdevs'].remove(attached_hostdev_info)
+
+            return attached_hostdev_info is not None
+
         LOG.error(
             "Trying to detach an unsupported device type."
             "The fakelibvirt implementation is incomplete "
@@ -1409,6 +1500,11 @@ class Domain(object):
         pass
 
     def XMLDesc(self, flags):
+        loader = '<loader/>'
+        if self._def['os'].get('loader_stateless'):
+            loader = ('<loader stateless="%s"/>' %
+                self._def['os'].get('loader_stateless'))
+
         disks = ''
         for disk in self._def['devices']['disks']:
             if disk['type'] == 'file':
@@ -1416,12 +1512,30 @@ class Domain(object):
             else:
                 source_attr = 'dev'
 
-            disks += '''<disk type='%(type)s' device='%(device)s'>
+            strformat = """
+    <disk type='%(type)s' device='%(device)s'>
       <driver name='%(driver_name)s' type='%(driver_type)s'/>
-      <source %(source_attr)s='%(source)s'/>
+      <source %(source_attr)s='%(source)s'"""
+            if 'encryption_format' not in disk:
+                strformat += '/>'
+            else:
+                strformat += """>
+        <encryption format='%(encryption_format)s'>
+          <secret type='passphrase' uuid='%(encryption_secret)s'/>
+        </encryption>
+      </source>"""
+            strformat += """
       <target dev='%(target_dev)s' bus='%(target_bus)s'/>
       <address type='drive' controller='0' bus='0' unit='0'/>
-    </disk>''' % dict(source_attr=source_attr, **disk)
+    </disk>"""
+            disks += strformat % dict(source_attr=source_attr, **disk)
+        filesystems = ''
+        for filesystem in self._def['devices']['filesystem']:
+            filesystems += '''<filesystem type='%(type)s'>
+      <driver type='%(driver_type)s'/>
+      <source dir='%(source)s'/>
+      <target dir='%(target)s'/>
+    </filesystem>''' % dict(source_attr=source_attr, **filesystem)
         nics = ''
         for func, nic in enumerate(self._def['devices']['nics']):
             if func > 7:
@@ -1475,10 +1589,18 @@ class Domain(object):
 
         hostdevs = ''
         for hostdev in self._def['devices']['hostdevs']:
-            hostdevs += '''<hostdev mode='subsystem' type='%(type)s' model='%(model)s'>
+            if hostdev['type'] == 'mdev':
+                hostdevs += '''<hostdev mode='subsystem' type='%(type)s' model='%(model)s'>
     <source>
       <address uuid='%(address_uuid)s'/>
     </source>
+    </hostdev>
+            ''' % hostdev  # noqa
+            if hostdev['type'] == 'pci':
+                hostdevs += '''<hostdev mode='subsystem' type='%(type)s' managed='%(managed)s'>
+    <source>
+      <address domain='%(domain)s' bus='%(bus)s' slot='%(slot)s' function='%(function)s'/>
+    </source>
     </hostdev>
             ''' % hostdev  # noqa
 
@@ -1525,14 +1647,19 @@ class Domain(object):
                 <source host="-1" service="-1" mode="bind"/>
                 </serial>"""
 
+        vcpuset = ''
+        if self._def['vcpu'].get('cpuset'):
+            vcpuset = ' cpuset="' + self._def['vcpu']['cpuset'] + '"'
+
         return '''<domain type='kvm'>
   <name>%(name)s</name>
   <uuid>%(uuid)s</uuid>
   <memory>%(memory)s</memory>
   <currentMemory>%(memory)s</currentMemory>
-  <vcpu>%(vcpu)s</vcpu>
+  <vcpu%(vcpuset)s>%(vcpu)s</vcpu>
   <os>
     <type arch='%(arch)s' machine='pc-0.12'>hvm</type>
+    %(loader)s
     <boot dev='hd'/>
   </os>
   <features>
@@ -1549,6 +1676,7 @@ class Domain(object):
   <devices>
     <emulator>/usr/bin/kvm</emulator>
     %(disks)s
+    %(filesystems)s
     <controller type='ide' index='0'>
       <address type='pci' domain='0x0000' bus='0x00' slot='0x01'
                function='0x1'/>
@@ -1578,9 +1706,12 @@ class Domain(object):
 </domain>''' % {'name': self._def['name'],
                 'uuid': self._def['uuid'],
                 'memory': self._def['memory'],
-                'vcpu': self._def['vcpu'],
+                'vcpuset': vcpuset,
+                'vcpu': self._def['vcpu']['number'],
                 'arch': self._def['os']['arch'],
+                'loader': loader,
                 'disks': disks,
+                'filesystems': filesystems,
                 'nics': nics,
                 'hostdevs': hostdevs,
                 'vpmems': vpmems,
@@ -1610,7 +1741,7 @@ class Domain(object):
 
     def vcpus(self):
         vcpus = ([], [])
-        for i in range(0, self._def['vcpu']):
+        for i in range(0, self._def['vcpu']['number']):
             vcpus[0].append((i, 1, 120405, i))
             vcpus[1].append((True, True, True, True))
         return vcpus
@@ -1700,6 +1831,11 @@ class Secret(object):
         tree = etree.fromstring(xml)
         self._uuid = tree.find('./uuid').text
         self._private = tree.get('private') == 'yes'
+        self._usage_id = None
+        usage = tree.find('./usage')
+        if usage is not None:
+            if usage.get('type') == 'volume':
+                self._usage_id = usage.find('volume').text
 
     def setValue(self, value, flags=0):
         self._value = value
@@ -1726,6 +1862,14 @@ class Secret(object):
     def undefine(self):
         self._connection._remove_secret(self)
 
+    def UUIDString(self):
+        if self._uuid is not None:
+            return self._uuid
+
+    def usageID(self):
+        if self._usage_id is not None:
+            return self._usage_id
+
 
 class Connection(object):
     def __init__(
@@ -1906,7 +2050,8 @@ class Connection(object):
         if arch == 'x86_64':
             aliases = {'pc': 'pc-i440fx-2.11', 'q35': 'pc-q35-2.11'}
             return fake_libvirt_data.DOMCAPABILITIES_X86_64_TEMPLATE % \
-                {'features': self._domain_capability_features,
+                {'devices': self._domain_capability_devices,
+                 'features': self._domain_capability_features,
                  'mtype': aliases.get(machine_type, machine_type)}
 
         raise Exception("fakelibvirt doesn't support getDomainCapabilities "
@@ -1973,6 +2118,119 @@ class Connection(object):
         }
         return mapping.get(arch, [])
 
+    _domain_capability_devices_default = '''    <disk supported='yes'>
+      <enum name='diskDevice'>
+        <value>disk</value>
+        <value>cdrom</value>
+        <value>floppy</value>
+        <value>lun</value>
+      </enum>
+      <enum name='bus'>
+        <value>ide</value>
+        <value>fdc</value>
+        <value>scsi</value>
+        <value>virtio</value>
+        <value>usb</value>
+        <value>sata</value>
+      </enum>
+    </disk>
+    <graphics supported='yes'>
+      <enum name='type'>
+        <value>sdl</value>
+        <value>vnc</value>
+        <value>spice</value>
+      </enum>
+    </graphics>
+    <video supported='yes'>
+      <enum name='modelType'>
+        <value>vga</value>
+        <value>cirrus</value>
+        <value>vmvga</value>
+        <value>qxl</value>
+        <value>virtio</value>
+      </enum>
+    </video>
+    <hostdev supported='yes'>
+      <enum name='mode'>
+        <value>subsystem</value>
+      </enum>
+      <enum name='startupPolicy'>
+        <value>default</value>
+        <value>mandatory</value>
+        <value>requisite</value>
+        <value>optional</value>
+      </enum>
+      <enum name='subsysType'>
+        <value>usb</value>
+        <value>pci</value>
+        <value>scsi</value>
+      </enum>
+      <enum name='capsType'/>
+      <enum name='pciBackend'>
+        <value>default</value>
+        <value>vfio</value>
+      </enum>
+    </hostdev>'''
+
+    _domain_capability_devices_with_tpm_supported = \
+        _domain_capability_devices_default + '''
+    <tpm supported='yes'>
+      <enum name='model'>
+        <value>tpm-tis</value>
+        <value>tpm-crb</value>
+      </enum>
+      <enum name='backendModel'>
+        <value>passthrough</value>
+        <value>emulator</value>
+        <value>external</value>
+      </enum>
+    </tpm>
+'''
+
+    # TODO(tkajinam): Merge this to the default value once libvirt >= 8.0.0 is
+    # required
+    _domain_capability_devices_with_tpm_unsupported = \
+        _domain_capability_devices_default + '''
+    <tpm supported='no'/>
+'''
+
+    _domain_capability_devices_with_no_emulator = \
+        _domain_capability_devices_default + '''
+    <tpm supported='yes'>
+      <enum name='model'>
+        <value>tpm-tis</value>
+        <value>tpm-crb</value>
+      </enum>
+      <enum name='backendModel'>
+        <value>passthrough</value>
+        <value>external</value>
+      </enum>
+    </tpm>
+'''
+
+    # TODO(tkajinam): Merge this to _supported value once libvirt >= 8.6.0 is
+    # required
+    _domain_capability_devices_with_tpm_versions = \
+        _domain_capability_devices_default + '''
+    <tpm supported='yes'>
+      <enum name='model'>
+        <value>tpm-tis</value>
+      </enum>
+      <enum name='backendModel'>
+        <value>passthrough</value>
+        <value>emulator</value>
+        <value>external</value>
+      </enum>
+      <enum name='backendVersion'>
+        <value>2.0</value>
+      </enum>
+    </tpm>
+'''
+
+    # Devices are kept separately so that the tests can patch this
+    # class variable with alternate values.
+    _domain_capability_devices = _domain_capability_devices_default
+
     # Features are kept separately so that the tests can patch this
     # class variable with alternate values.
     _domain_capability_features = '''  <features>
@@ -1990,6 +2248,16 @@ class Connection(object):
     _domain_capability_features_with_SEV_unsupported = \
         _domain_capability_features_with_SEV.replace('yes', 'no')
 
+    _domain_capability_features_with_SEV_max_guests = '''  <features>
+    <gic supported='no'/>
+    <sev supported='yes'>
+      <cbitpos>47</cbitpos>
+      <reducedPhysBits>1</reducedPhysBits>
+      <maxGuests>100</maxGuests>
+      <maxESGuests>15</maxESGuests>
+    </sev>
+  </features>'''
+
     def getCapabilities(self):
         """Return spoofed capabilities."""
         numa_topology = self.host_info.numa_topology
@@ -2044,6 +2312,12 @@ class Connection(object):
 
         return VIR_CPU_COMPARE_IDENTICAL
 
+    def compareHypervisorCPU(
+        self, emulator, arch, machine, virttype,
+        xml, flags
+    ):
+        return self.compareCPU(xml, flags)
+
     def getCPUStats(self, cpuNum, flag):
         if cpuNum < 2:
             return {'kernel': 5664160000000,
@@ -2061,6 +2335,17 @@ class Connection(object):
         return self.pci_info.get_device_by_name(dev_name)
 
     def nodeDeviceLookupByName(self, name):
+        # See bug https://bugs.launchpad.net/nova/+bug/2098892
+        # We don't test this by importing the libvirt module because the
+        # libvirt module is forbidden to be imported into our test
+        # environment. It is excluded from test-requirements.txt and we
+        # also use the ImportModulePoisonFixture in nova/test.py to prevent
+        # use of modules such as libvirt.
+        if not isinstance(name, str) and name is not None:
+            raise TypeError(
+                'virNodeDeviceLookupByName() argument 2 must be str or '
+                f'None, not {type(name)}')
+
         if name.startswith('mdev'):
             return self.mdev_info.get_device_by_name(name)
 
@@ -2079,6 +2364,12 @@ class Connection(object):
                     error_code=VIR_ERR_NO_NODE_DEVICE,
                     error_domain=VIR_FROM_NODEDEV)
 
+    def nodeDeviceCreateXML(self, xml, flags):
+        pass
+
+    def nodeDeviceDefineXML(self, xml, flags):
+        pass
+
     def listDevices(self, cap, flags):
         if cap == 'pci':
             return self.pci_info.get_all_devices()
@@ -2112,8 +2403,15 @@ class Connection(object):
                     <feature policy='require' name='aes'/>
                   </cpu>"""
 
+    def listAllSecrets(self, flags):
+        return [secret for secret in self._secrets.values()]
+
     def secretLookupByUsage(self, usage_type_obj, usage_id):
-        pass
+        for secret in self._secrets.values():
+            # Ignore usage_type_obj because we don't have a way to map libvrt
+            # usage type constants to strings.
+            if secret._usage_id == usage_id:
+                return secret
 
     def secretDefineXML(self, xml):
         secret = Secret(self, xml)
@@ -2141,10 +2439,10 @@ class Connection(object):
 
 def openAuth(uri, auth, flags=0):
 
-    if type(auth) != list:
+    if type(auth) is not list:
         raise Exception("Expected a list for 'auth' parameter")
 
-    if type(auth[0]) != list:
+    if type(auth[0]) is not list:
         raise Exception("Expected a function in 'auth[0]' parameter")
 
     if not callable(auth[1]):
@@ -2278,6 +2576,8 @@ class LibvirtFixture(fixtures.Fixture):
             'Linux', '', '5.4.0-0-generic', '', obj_fields.Architecture.X86_64)
         self.mock_uname = self.useFixture(
             fixtures.MockPatch('os.uname', return_value=fake_uname)).mock
+        self.useFixture(fixtures.MonkeyPatch(
+            'nova.virt.libvirt.driver.sys.platform', 'linux'))
 
         real_exists = os.path.exists
 
