@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-import six.moves.urllib.parse as urlparse
+from unittest import mock
+from urllib import parse as urlparse
+
 import webob
 
 from nova.api.openstack import common
@@ -913,12 +914,12 @@ class DisabledFlavorsWithRealDBTestV21(test.TestCase):
         self.context = self.req.environ['nova.context']
         self.admin_context = context.get_admin_context()
 
-        self.disabled_type = self._create_disabled_instance_type()
+        self.disabled_type = self._create_disabled_flavor()
         self.addCleanup(self.disabled_type.destroy)
-        self.inst_types = objects.FlavorList.get_all(self.admin_context)
+        self.flavors = objects.FlavorList.get_all(self.admin_context)
         self.controller = self.Controller()
 
-    def _create_disabled_instance_type(self):
+    def _create_disabled_flavor(self):
         flavor = objects.Flavor(context=self.admin_context,
                                 name='foo.disabled', flavorid='10.disabled',
                                 memory_mb=512, vcpus=2, root_gb=1,
@@ -934,7 +935,7 @@ class DisabledFlavorsWithRealDBTestV21(test.TestCase):
         flavor_list = self.controller.index(self.req)['flavors']
         api_flavorids = set(f['id'] for f in flavor_list)
 
-        db_flavorids = set(i['flavorid'] for i in self.inst_types)
+        db_flavorids = set(i['flavorid'] for i in self.flavors)
         disabled_flavorid = str(self.disabled_type['flavorid'])
 
         self.assertIn(disabled_flavorid, db_flavorids)
@@ -947,7 +948,7 @@ class DisabledFlavorsWithRealDBTestV21(test.TestCase):
         flavor_list = self.controller.index(self.req)['flavors']
         api_flavorids = set(f['id'] for f in flavor_list)
 
-        db_flavorids = set(i['flavorid'] for i in self.inst_types)
+        db_flavorids = set(i['flavorid'] for i in self.flavors)
         disabled_flavorid = str(self.disabled_type['flavorid'])
 
         self.assertIn(disabled_flavorid, db_flavorids)
