@@ -19,9 +19,9 @@ and as a WSGI layer
 """
 
 import copy
+from unittest import mock
+from urllib import parse as urlparse
 
-import mock
-import six.moves.urllib.parse as urlparse
 import webob
 
 from nova.api.openstack.compute import images as images_v21
@@ -42,9 +42,8 @@ IMAGE_FIXTURES = image_fixtures.get_image_fixtures()
 class ImagesControllerTestV21(test.NoDBTestCase):
     """Test of the OpenStack API /images application controller w/Glance.
     """
-    image_controller_class = images_v21.ImagesController
-    url_base = '/v2/fake'
-    bookmark_base = '/fake'
+    url_base = '/v2/%s' % fakes.FAKE_PROJECT_ID
+    bookmark_base = '/%s' % fakes.FAKE_PROJECT_ID
     http_request = fakes.HTTPRequestV21
 
     def setUp(self):
@@ -53,10 +52,10 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         self.flags(api_servers=['http://localhost:9292'], group='glance')
         fakes.stub_out_networking(self)
         fakes.stub_out_key_pair_funcs(self)
-        fakes.stub_out_compute_api_snapshot(self.stubs)
-        fakes.stub_out_compute_api_backup(self.stubs)
+        fakes.stub_out_compute_api_snapshot(self)
+        fakes.stub_out_compute_api_backup(self)
 
-        self.controller = self.image_controller_class()
+        self.controller = images_v21.ImagesController()
         self.url_prefix = "http://localhost%s/images" % self.url_base
         self.bookmark_prefix = "http://localhost%s/images" % self.bookmark_base
         self.uuid = 'fa95aaf5-ab3b-4cd8-88c0-2be7dd051aaf'
@@ -79,6 +78,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
                       'minDisk': 10,
                       'progress': 100,
                       'minRam': 128,
+                      'OS-EXT-IMG-SIZE:size': 25165824,
                       "links": [{
                                     "rel": "self",
                                     "href": "%s/123" % self.url_prefix
@@ -92,7 +92,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
                                     "rel": "alternate",
                                     "type": "application/vnd.openstack.image",
                                     "href": self.alternate %
-                                            (glance.generate_glance_url(),
+                                            (glance.generate_glance_url('ctx'),
                                              123),
                                 }],
             },
@@ -111,6 +111,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
                       'progress': 25,
                       'minDisk': 0,
                       'minRam': 0,
+                      'OS-EXT-IMG-SIZE:size': 25165824,
                       'server': {
                           'id': self.server_uuid,
                           "links": [{
@@ -136,13 +137,13 @@ class ImagesControllerTestV21(test.NoDBTestCase):
                                     "type":
                                         "application/vnd.openstack.image",
                                     "href": self.alternate %
-                                            (glance.generate_glance_url(),
+                                            (glance.generate_glance_url('ctx'),
                                              124),
                                 }],
             },
         }
 
-    @mock.patch('nova.image.api.API.get', return_value=IMAGE_FIXTURES[0])
+    @mock.patch('nova.image.glance.API.get', return_value=IMAGE_FIXTURES[0])
     def test_get_image(self, get_mocked):
         request = self.http_request.blank(self.url_base + 'images/123')
         actual_image = self.controller.show(request, '123')
@@ -150,10 +151,11 @@ class ImagesControllerTestV21(test.NoDBTestCase):
                         matchers.DictMatches(self.expected_image_123))
         get_mocked.assert_called_once_with(mock.ANY, '123')
 
-    @mock.patch('nova.image.api.API.get', return_value=IMAGE_FIXTURES[1])
+    @mock.patch('nova.image.glance.API.get', return_value=IMAGE_FIXTURES[1])
     def test_get_image_with_custom_prefix(self, _get_mocked):
-        self.flags(osapi_compute_link_prefix='https://zoo.com:42',
-                   osapi_glance_link_prefix='http://circus.com:34')
+        self.flags(compute_link_prefix='https://zoo.com:42',
+                   glance_link_prefix='http://circus.com:34',
+                   group='api')
         fake_req = self.http_request.blank(self.url_base + 'images/124')
         actual_image = self.controller.show(fake_req, '124')
 
@@ -173,14 +175,14 @@ class ImagesControllerTestV21(test.NoDBTestCase):
 
         self.assertThat(actual_image, matchers.DictMatches(expected_image))
 
-    @mock.patch('nova.image.api.API.get',
+    @mock.patch('nova.image.glance.API.get',
                 side_effect=exception.ImageNotFound(image_id=''))
     def test_get_image_404(self, _get_mocked):
         fake_req = self.http_request.blank(self.url_base + 'images/unknown')
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller.show, fake_req, 'unknown')
 
-    @mock.patch('nova.image.api.API.get_all', return_value=IMAGE_FIXTURES)
+    @mock.patch('nova.image.glance.API.get_all', return_value=IMAGE_FIXTURES)
     def test_get_image_details(self, get_all_mocked):
         request = self.http_request.blank(self.url_base + 'images/detail')
         response = self.controller.detail(request)
@@ -195,7 +197,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_125["links"][0]["href"] = "%s/125" % self.url_prefix
         image_125["links"][1]["href"] = "%s/125" % self.bookmark_prefix
         image_125["links"][2]["href"] = (
-            "%s/images/125" % glance.generate_glance_url())
+            "%s/images/125" % glance.generate_glance_url('ctx'))
 
         image_126 = copy.deepcopy(self.expected_image_124["image"])
         image_126['id'] = '126'
@@ -205,7 +207,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_126["links"][0]["href"] = "%s/126" % self.url_prefix
         image_126["links"][1]["href"] = "%s/126" % self.bookmark_prefix
         image_126["links"][2]["href"] = (
-            "%s/images/126" % glance.generate_glance_url())
+            "%s/images/126" % glance.generate_glance_url('ctx'))
 
         image_127 = copy.deepcopy(self.expected_image_124["image"])
         image_127['id'] = '127'
@@ -215,7 +217,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_127["links"][0]["href"] = "%s/127" % self.url_prefix
         image_127["links"][1]["href"] = "%s/127" % self.bookmark_prefix
         image_127["links"][2]["href"] = (
-            "%s/images/127" % glance.generate_glance_url())
+            "%s/images/127" % glance.generate_glance_url('ctx'))
 
         image_128 = copy.deepcopy(self.expected_image_124["image"])
         image_128['id'] = '128'
@@ -225,7 +227,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_128["links"][0]["href"] = "%s/128" % self.url_prefix
         image_128["links"][1]["href"] = "%s/128" % self.bookmark_prefix
         image_128["links"][2]["href"] = (
-            "%s/images/128" % glance.generate_glance_url())
+            "%s/images/128" % glance.generate_glance_url('ctx'))
 
         image_129 = copy.deepcopy(self.expected_image_124["image"])
         image_129['id'] = '129'
@@ -235,7 +237,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_129["links"][0]["href"] = "%s/129" % self.url_prefix
         image_129["links"][1]["href"] = "%s/129" % self.bookmark_prefix
         image_129["links"][2]["href"] = (
-            "%s/images/129" % glance.generate_glance_url())
+            "%s/images/129" % glance.generate_glance_url('ctx'))
 
         image_130 = copy.deepcopy(self.expected_image_123["image"])
         image_130['id'] = '130'
@@ -246,7 +248,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_130["links"][0]["href"] = "%s/130" % self.url_prefix
         image_130["links"][1]["href"] = "%s/130" % self.bookmark_prefix
         image_130["links"][2]["href"] = (
-            "%s/images/130" % glance.generate_glance_url())
+            "%s/images/130" % glance.generate_glance_url('ctx'))
 
         image_131 = copy.deepcopy(self.expected_image_123["image"])
         image_131['id'] = '131'
@@ -257,7 +259,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         image_131["links"][0]["href"] = "%s/131" % self.url_prefix
         image_131["links"][1]["href"] = "%s/131" % self.bookmark_prefix
         image_131["links"][2]["href"] = (
-            "%s/images/131" % glance.generate_glance_url())
+            "%s/images/131" % glance.generate_glance_url('ctx'))
 
         expected = [self.expected_image_123["image"],
                     self.expected_image_124["image"],
@@ -267,14 +269,14 @@ class ImagesControllerTestV21(test.NoDBTestCase):
 
         self.assertThat(expected, matchers.DictListMatches(response_list))
 
-    @mock.patch('nova.image.api.API.get_all')
+    @mock.patch('nova.image.glance.API.get_all')
     def test_get_image_details_with_limit(self, get_all_mocked):
         request = self.http_request.blank(self.url_base +
                                           'images/detail?limit=2')
         self.controller.detail(request)
         get_all_mocked.assert_called_once_with(mock.ANY, limit=2, filters={})
 
-    @mock.patch('nova.image.api.API.get_all')
+    @mock.patch('nova.image.glance.API.get_all')
     def test_get_image_details_with_limit_and_page_size(self, get_all_mocked):
         request = self.http_request.blank(
             self.url_base + 'images/detail?limit=2&page_size=1')
@@ -282,7 +284,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         get_all_mocked.assert_called_once_with(mock.ANY, limit=2, filters={},
                                                page_size=1)
 
-    @mock.patch('nova.image.api.API.get_all')
+    @mock.patch('nova.image.glance.API.get_all')
     def _detail_request(self, filters, request, get_all_mocked):
         self.controller.detail(request)
         get_all_mocked.assert_called_once_with(mock.ANY, filters=filters)
@@ -341,7 +343,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         request = self.http_request.blank(self.url_base + 'images/detail')
         self._detail_request(filters, request)
 
-    @mock.patch('nova.image.api.API.get_all', side_effect=exception.Invalid)
+    @mock.patch('nova.image.glance.API.get_all', side_effect=exception.Invalid)
     def test_image_detail_invalid_marker(self, _get_all_mocked):
         request = self.http_request.blank(self.url_base + '?marker=invalid')
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.detail,
@@ -351,22 +353,19 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         view = images_view.ViewBuilder()
         request = self.http_request.blank(self.url_base + 'images/1')
         generated_url = view._get_alternate_link(request, 1)
-        actual_url = "%s/images/1" % glance.generate_glance_url()
+        actual_url = "%s/images/1" % glance.generate_glance_url('ctx')
         self.assertEqual(generated_url, actual_url)
 
-    def _check_response(self, controller_method, response, expected_code):
-        self.assertEqual(expected_code, controller_method.wsgi_code)
-
-    @mock.patch('nova.image.api.API.delete')
+    @mock.patch('nova.image.glance.API.delete')
     def test_delete_image(self, delete_mocked):
         request = self.http_request.blank(self.url_base + 'images/124')
         request.method = 'DELETE'
         delete_method = self.controller.delete
-        response = delete_method(request, '124')
-        self._check_response(delete_method, response, 204)
+        delete_method(request, '124')
+        self.assertEqual(204, delete_method.wsgi_codes(request))
         delete_mocked.assert_called_once_with(mock.ANY, '124')
 
-    @mock.patch('nova.image.api.API.delete',
+    @mock.patch('nova.image.glance.API.delete',
                 side_effect=exception.ImageNotAuthorized(image_id='123'))
     def test_delete_deleted_image(self, _delete_mocked):
         # If you try to delete a deleted image, you get back 403 Forbidden.
@@ -375,7 +374,7 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                           request, '123')
 
-    @mock.patch('nova.image.api.API.delete',
+    @mock.patch('nova.image.glance.API.delete',
                 side_effect=exception.ImageNotFound(image_id='123'))
     def test_delete_image_not_found(self, _delete_mocked):
         request = self.http_request.blank(self.url_base + 'images/300')
@@ -383,7 +382,8 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller.delete, request, '300')
 
-    @mock.patch('nova.image.api.API.get_all', return_value=[IMAGE_FIXTURES[0]])
+    @mock.patch('nova.image.glance.API.get_all',
+                return_value=[IMAGE_FIXTURES[0]])
     def test_get_image_next_link(self, get_all_mocked):
         request = self.http_request.blank(
             self.url_base + 'imagesl?limit=1')
@@ -395,7 +395,8 @@ class ImagesControllerTestV21(test.NoDBTestCase):
         self.assertThat({'limit': ['1'], 'marker': [IMAGE_FIXTURES[0]['id']]},
                         matchers.DictMatches(params))
 
-    @mock.patch('nova.image.api.API.get_all', return_value=[IMAGE_FIXTURES[0]])
+    @mock.patch('nova.image.glance.API.get_all',
+                return_value=[IMAGE_FIXTURES[0]])
     def test_get_image_details_next_link(self, get_all_mocked):
         request = self.http_request.blank(
             self.url_base + 'images/detail?limit=1')
