@@ -12,10 +12,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import fixtures as fx
-import mock
 from oslo_utils.fixture import uuidsentinel as uuids
-import six
 
 from nova.api.openstack.compute import server_external_events \
                                                  as server_external_events_v21
@@ -193,7 +193,7 @@ class ServerExternalEventsTestV21(test.NoDBTestCase):
                           self.api.create, self.req, body=body)
 
     def test_create_unknown_events(self):
-        self.event_1['name'] = 'unkown_event'
+        self.event_1['name'] = 'unknown_event'
         body = {'events': self.event_1}
         self.assertRaises(self.invalid_error,
                           self.api.create, self.req, body=body)
@@ -260,8 +260,7 @@ class ServerExternalEventsTestV276(ServerExternalEventsTestV21):
                 self.api.create,
                 req,
                 body=body)
-        self.assertIn('Invalid input for field/attribute name.',
-                      six.text_type(exp))
+        self.assertIn('Invalid input for field/attribute name.', str(exp))
 
 
 @mock.patch('nova.objects.InstanceMappingList.get_by_instance_uuids',
