@@ -375,7 +375,7 @@ notification_object_data = {
     'ComputeTaskNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
     'ComputeTaskPayload': '1.0-59a9b78b6199470a83c8d07bffd13f5e',
     'DestinationPayload': '1.0-d8faf610201bf5f460892243f6632a37',
-    'EventType': '1.21-6a5f57fafe478f354f66b81b4cb537ea',
+    'EventType': '1.22-fab37f98c6e7513edece32ad5420f549',
     'ExceptionNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
     'ExceptionPayload': '1.1-34f006107693b8c9eaf4b104157d21b4',
     'FlavorNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
@@ -385,37 +385,39 @@ notification_object_data = {
     # ImageMetaProps, so when you see a fail here for that reason, you must
     # *also* bump the version of ImageMetaPropsPayload. See its docstring for
     # more information.
-    'ImageMetaPropsPayload': '1.14-af9efd17cd034596be792fa25f24e83d',
+    'ImageMetaPropsPayload': '1.15-a3d7a75fedbf373bcab2d9d6ece249df',
     'InstanceActionNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionPayload': '1.8-b948818df6ec562e4eb4b23e515e451b',
+    'InstanceActionPayload': '1.9-525dcf81b6e4592d935712a2675309dc',
     'InstanceActionRebuildNotification':
         '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionRebuildPayload': '1.9-3c1ac23f70c03665797f792ace3d9176',
+    'InstanceActionRebuildPayload': '1.10-b741a7add29a8c1f584fd5d5b3362372',
     'InstanceActionRescueNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionRescuePayload': '1.3-329dda4c0a516c0fedc68aa02ccf3e88',
+    'InstanceActionRescuePayload': '1.4-29e0327c2ad6953d367db68d9aceef20',
     'InstanceActionResizePrepNotification':
         '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionResizePrepPayload': '1.3-8969438a48c496569c2add19b170dca1',
+    'InstanceActionResizePrepPayload': '1.4-214cb6123beb0ffe5eb1e08ce424086b',
+    'InstanceActionShareNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
+    'InstanceActionSharePayload': '1.0-6dafbbd061f582e9c0ce100eeee790b9',
     'InstanceActionVolumeNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionVolumePayload': '1.6-b5fd2b23dbafe33b72cdbbb7b937bf18',
+    'InstanceActionVolumePayload': '1.7-542d9e1485c6b925958df9912dc744ec',
     'InstanceActionVolumeSwapNotification':
     '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionVolumeSwapPayload': '1.8-9d8edd0a3556a69f0bfe17ab2641ca19',
+    'InstanceActionVolumeSwapPayload': '1.9-dc4e9852819bc3283b0c809a1f6602d4',
     'InstanceCreateNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceCreatePayload': '1.12-bd007423bfdb634448ec589b28f7cd39',
-    'InstancePayload': '1.8-0bfe0b258ae4cf6ff68463c9e75adb80',
+    'InstanceCreatePayload': '1.13-95a394da335944f074ac1bf58696ad87',
+    'InstancePayload': '1.9-164048a4d38ed9a4b2f9c3927f1c9038',
     'InstanceActionSnapshotNotification':
         '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceActionSnapshotPayload': '1.9-75809b8ef913f7d2e262c5c804213ac8',
+    'InstanceActionSnapshotPayload': '1.10-7a4878ef67da0939ee069bb78eab07ae',
     'InstanceExistsNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceExistsPayload': '2.0-5eb1efc3d1b03d87be82062219adaf6d',
+    'InstanceExistsPayload': '2.1-264c20f9c5d76320c9129865b9bc6f8f',
     'InstanceNUMACellPayload': '1.2-93c3cadf4f6ee427664c00039d0a1fed',
     'InstanceNUMATopologyPayload': '1.0-c7885d5da4c66525f20e7d637a7267c4',
     'InstancePCIRequestPayload': '1.1-ee482d38f6c84bb77bba3f4c8708231c',
     'InstancePCIRequestsPayload': '1.0-29cb0fa1ba6756423a9606437c3f247f',
     'InstanceStateUpdatePayload': '1.0-9d136730749479042d47f2a08c2c2e69',
     'InstanceUpdateNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
-    'InstanceUpdatePayload': '2.0-f6edbd2b53373f120debf5570ec14676',
+    'InstanceUpdatePayload': '2.1-25c1e9c521bd350c20cbda94b7cc7dc0',
     'IpPayload': '1.0-a9f0b3f1ef17ab12378096816faa4e55',
     'KeypairNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
     'KeypairPayload': '1.0-84f5e6da739af31ea10b89858c67b512',
@@ -432,6 +434,7 @@ notification_object_data = {
     'ServerGroupPayload': '1.1-f866cf95bf64bfa799193807e66a6a1e',
     'ServiceStatusNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
     'ServiceStatusPayload': '1.1-64c5f80b3865832b936898f11dcddad0',
+    'SharePayload': '1.0-80797287e01668692ee9beb70f269821',
     'VirtCPUTopologyPayload': '1.0-24fa607e20f48c829953edad6b0b5cc8',
     'VolumeUsageNotification': '1.0-a73147b93b520ff0061865849d3dfa56',
     'VolumeUsagePayload': '1.0-a12c300f0ddbc7488685c9d9ed36d9e8',
