@@ -13,8 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import collections
-import copy
-import ddt
 from unittest import mock
 
 from oslo_config import cfg
@@ -513,6 +511,20 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                     "remote_managed": "false",
                 }
             ),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b4",
+                    "product_id": "102e",
+                    "live_migratable": "true",
+                }
+            ),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b5",
+                    "product_id": "102f",
+                    "live_migratable": "false",
+                }
+            ),
         ]
         self.flags(device_spec=device_spec, group="pci")
         dev_filter = whitelist.Whitelist(device_spec)
@@ -595,6 +607,40 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                     'parent_addr': '0000:0e:00.0',
                     'numa_node': 0}))
 
+        # live_migratable set to true
+        # Warning: 'extra_info' field is set by the init or create method
+        self.live_migratable = []
+        self.live_migratable.append(
+            objects.PciDevice.create(
+                None, {
+                    'compute_node_id': 1,
+                    'address': '0000:0f:00.1',
+                    'vendor_id': '15b4',
+                    'product_id': '102e',
+                    'status': 'available',
+                    'request_id': None,
+                    'dev_type': fields.PciDeviceType.SRIOV_VF,
+                    'parent_addr': '0000:0f:00.0',
+                    'numa_node': 0,
+                }))
+
+        # live_migratable set to false
+        # Warning: 'extra_info' field is set by the init or create method
+        self.non_live_migratable = []
+        self.non_live_migratable.append(
+            objects.PciDevice.create(
+                None, {
+                    'compute_node_id': 1,
+                    'address': '0000:10:00.1',
+                    'vendor_id': '15b5',
+                    'product_id': '102f',
+                    'status': 'available',
+                    'request_id': None,
+                    'dev_type': fields.PciDeviceType.SRIOV_VF,
+                    'parent_addr': '0000:10:00.0',
+                    'numa_node': 0,
+                }))
+
         for dev in self.pci_tagged_devices:
             self.pci_stats.add_device(dev)
 
@@ -607,6 +653,12 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         for dev in self.locally_managed_netdevs:
             self.pci_stats.add_device(dev)
 
+        for dev in self.live_migratable:
+            self.pci_stats.add_device(dev)
+
+        for dev in self.non_live_migratable:
+            self.pci_stats.add_device(dev)
+
     def _assertPoolContent(self, pool, vendor_id, product_id, count, **tags):
         self.assertEqual(vendor_id, pool['vendor_id'])
         self.assertEqual(product_id, pool['product_id'])
@@ -620,8 +672,10 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         nr_untagged = len(self.pci_untagged_devices)
         nr_remote = len(self.remote_managed_netdevs)
         nr_local = len(self.locally_managed_netdevs)
+        nr_lm = len(self.live_migratable)
+        nr_nlm = len(self.non_live_migratable)
         self.assertEqual(
-            nr_tagged + nr_untagged + nr_remote + nr_local,
+            nr_tagged + nr_untagged + nr_remote + nr_local + nr_lm + nr_nlm,
             len(self.pci_stats.pools),
         )
         # Pools are ordered based on the number of keys. 'product_id',
@@ -679,6 +733,32 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         self.assertEqual(self.locally_managed_netdevs, devs)
         j += nr_local
 
+        # one live_migratable
+        devs = []
+        for i in range(j, j + nr_lm):
+            self._assertPoolContent(
+                self.pci_stats.pools[i],
+                "15b4",
+                "102e",
+                1,
+            )
+            devs += self.pci_stats.pools[i]['devices']
+        self.assertEqual(self.live_migratable, devs)
+        j += nr_lm
+
+        # one non_live_migratable
+        devs = []
+        for i in range(j, j + nr_nlm):
+            self._assertPoolContent(
+                self.pci_stats.pools[i],
+                "15b5",
+                "102f",
+                1,
+            )
+            devs += self.pci_stats.pools[i]['devices']
+        self.assertEqual(self.non_live_migratable, devs)
+        j += nr_nlm
+
     def test_add_devices(self):
         self._create_pci_devices()
         self._assertPools()
@@ -701,10 +781,19 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                         objects.InstancePCIRequest(count=1,
                             spec=[{'vendor_id': '15b3',
                                    'product_id': '101c',
-                                   PCI_REMOTE_MANAGED_TAG: 'False'}])]
+                                   PCI_REMOTE_MANAGED_TAG: 'False'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'vendor_id': '15b4',
+                                   'product_id': '102e',
+                                   'live_migratable': 'true'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'vendor_id': '15b5',
+                                   'product_id': '102f',
+                                   'live_migratable': 'false'}]),
+                        ]
         devs = self.pci_stats.consume_requests(pci_requests)
-        self.assertEqual(5, len(devs))
-        self.assertEqual(set(['0071', '0072', '101e', '101c']),
+        self.assertEqual(7, len(devs))
+        self.assertEqual(set(['0071', '0072', '101e', '101c', '102e', '102f']),
                          set([dev.product_id for dev in devs]))
         self._assertPoolContent(self.pci_stats.pools[0], '1137', '0072', 0)
         self._assertPoolContent(self.pci_stats.pools[1], '1137', '0072', 1)
@@ -725,6 +814,10 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                                 remote_managed='false')
         self._assertPoolContent(self.pci_stats.pools[9], '15b3', '101c', 0,
                                 remote_managed='false')
+        self._assertPoolContent(self.pci_stats.pools[10], '15b4', '102e', 0,
+                                live_migratable='true')
+        self._assertPoolContent(self.pci_stats.pools[11], '15b5', '102f', 0,
+                                live_migratable='false')
 
     def test_add_device_no_devspec(self):
         self._create_pci_devices()
@@ -1029,7 +1122,6 @@ class PciDeviceStatsPlacementSupportTestCase(test.NoDBTestCase):
         )
 
 
-@ddt.ddt
 class PciDeviceStatsProviderMappingTestCase(test.NoDBTestCase):
     def setUp(self):
         super().setUp()
@@ -1516,115 +1608,6 @@ class PciDeviceStatsProviderMappingTestCase(test.NoDBTestCase):
             },
         )
 
-    @mock.patch("nova.pci.stats.LOG.warning")
-    def _test_split_pool_rejected(self, consume_fn, mock_warning):
-        self.pci_stats = stats.PciDeviceStats(
-            objects.NUMATopology(), dev_filter=self.dev_filter
-        )
-        for vf_index in [1, 2]:
-            dev = objects.PciDevice(
-                compute_node_id=1,
-                vendor_id="dead",
-                product_id="beef",
-                address=f"0000:81:01.{vf_index}",
-                parent_addr="0000:81:01.0",
-                numa_node=0,
-                dev_type="type-VF",
-            )
-            self.pci_stats.add_device(dev)
-            dev.extra_info = {'rp_uuid': uuids.pf1}
-
-        self.pci_stats.populate_pools_metadata_from_assigned_devices()
-        # NOTE(gibi): this simulates an invalid situation where the same
-        # VFs from the same PF are split across multiple pools. This should
-        # not happen, but we want to check that the code rejects it
-        self.pci_stats.pools.extend(copy.deepcopy(self.pci_stats.pools))
-
-        vf_req = objects.InstancePCIRequest(
-            count=1,
-            alias_name='a-vf',
-            request_id=uuids.vf_req,
-            spec=[
-                {
-                    "vendor_id": "dead",
-                    "product_id": "beef",
-                    "dev_type": "type-VF",
-                    # Simulate that the scheduler already allocate a candidate
-                    # and the mapping is stored in the request.
-                    "rp_uuids": ",".join([uuids.pf1])
-                }
-            ],
-        )
-
-        self.assertRaises(
-            exception.PciDeviceRequestFailed,
-            consume_fn,
-            [vf_req],
-        )
-        mock_warning.assert_called_once_with(
-            'The PCI allocation logic assumes that devices related to the '
-            'same rp_uuid are in the same pool. However the following '
-            'rp_uuids are split across multiple pools. This should not '
-            'happen. Please file a bug report. %s',
-            {uuids.pf1: self.pci_stats.pools})
-
-    def test_consume_split_pool_rejected(self):
-        self._test_split_pool_rejected(
-            lambda reqs: self.pci_stats.consume_requests(reqs))
-
-    def test_apply_split_pool_rejected(self):
-        self._test_split_pool_rejected(
-            lambda reqs: self.pci_stats.apply_requests(reqs, {}))
-
-    @ddt.data(
-        # no pool no problem
-        [[], True],
-        # single pool without rp_uuid is OK
-        [[{}], True],
-        # single pool with rp_uuid is OK
-        [[{}, {"rp_uuid": uuids.pf1}], True],
-        # two pools with different rp_uuid is OK
-        [[{"rp_uuid": uuids.pf1}, {"rp_uuid": uuids.pf2}], True],
-        # two pools with the same rp_uuid is NOT OK
-        [[{}, {"rp_uuid": uuids.pf1}, {"rp_uuid": uuids.pf2},
-          {"rp_uuid": uuids.pf1}], False]
-    )
-    @ddt.unpack
-    def test_assert_one_pool_per_rp_uuid(self, pools, result):
-        self.assertEqual(
-            result, stats.PciDeviceStats._assert_one_pool_per_rp_uuid(pools))
-
-    def test_find_pool_ignores_metadata(self):
-        self.pci_stats = stats.PciDeviceStats(
-            objects.NUMATopology(), dev_filter=self.dev_filter
-        )
-        dev1 = objects.PciDevice(
-            compute_node_id=1,
-            vendor_id="dead",
-            product_id="beef",
-            address="0000:81:01.1",
-            parent_addr="0000:81:01.0",
-            numa_node=0,
-            dev_type="type-VF",
-        )
-        dev1.extra_info = {'rp_uuid': uuids.pf1}
-        self.pci_stats.add_device(dev1)
-        self.pci_stats.populate_pools_metadata_from_assigned_devices()
-
-        dev2 = objects.PciDevice(
-            compute_node_id=1,
-            vendor_id="dead",
-            product_id="beef",
-            address="0000:81:01.2",
-            parent_addr="0000:81:01.0",
-            numa_node=0,
-            dev_type="type-VF",
-        )
-        dev2.extra_info = {'rp_uuid': uuids.pf1}
-        self.pci_stats.add_device(dev2)
-
-        self.assertEqual(1, len(self.pci_stats.pools))
-
 
 class PciDeviceVFPFStatsTestCase(test.NoDBTestCase):
 
