@@ -10,9 +10,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
 
 from eventlet import tpool
-import mock
+from oslo_concurrency import processutils
 from oslo_serialization import jsonutils
 from oslo_utils.fixture import uuidsentinel as uuids
 
@@ -110,7 +111,7 @@ class RbdTestCase(test.NoDBTestCase):
         self.rbd_pool = 'rbd'
         self.rbd_connect_timeout = 5
         self.flags(
-            images_rbd_pool=self.images_rbd_pool,
+            images_rbd_pool=self.rbd_pool,
             images_rbd_ceph_conf='/foo/bar.conf',
             rbd_connect_timeout=self.rbd_connect_timeout,
             rbd_user='foo', group='libvirt')
@@ -118,13 +119,15 @@ class RbdTestCase(test.NoDBTestCase):
         rados_patcher = mock.patch.object(rbd_utils, 'rados')
         self.mock_rados = rados_patcher.start()
         self.addCleanup(rados_patcher.stop)
-        self.mock_rados.Rados = mock.Mock
-        self.mock_rados.Rados.ioctx = mock.Mock()
-        self.mock_rados.Rados.connect = mock.Mock()
-        self.mock_rados.Rados.shutdown = mock.Mock()
-        self.mock_rados.Rados.open_ioctx = mock.Mock()
-        self.mock_rados.Rados.open_ioctx.return_value = \
-            self.mock_rados.Rados.ioctx
+        self.mock_rados.Rados = mock.Mock()
+        self.rados_inst = mock.Mock()
+        self.mock_rados.Rados.return_value = self.rados_inst
+        self.rados_inst.ioctx = mock.Mock()
+        self.rados_inst.connect = mock.Mock()
+        self.rados_inst.shutdown = mock.Mock()
+        self.rados_inst.open_ioctx = mock.Mock()
+        self.rados_inst.open_ioctx.return_value = \
+            self.rados_inst.ioctx
         self.mock_rados.Error = Exception
 
         rbd_patcher = mock.patch.object(rbd_utils, 'rbd')
@@ -338,33 +341,31 @@ class RbdTestCase(test.NoDBTestCase):
 
     def test_connect_to_rados_default(self):
         ret = self.driver._connect_to_rados()
-        self.mock_rados.Rados.connect.assert_called_once_with(
+        self.rados_inst.connect.assert_called_once_with(
                 timeout=self.rbd_connect_timeout)
-        self.assertTrue(self.mock_rados.Rados.open_ioctx.called)
-        self.assertIsInstance(ret[0], self.mock_rados.Rados)
-        self.assertEqual(self.mock_rados.Rados.ioctx, ret[1])
-        self.mock_rados.Rados.open_ioctx.assert_called_with(self.rbd_pool)
+        self.assertTrue(self.rados_inst.open_ioctx.called)
+        self.assertEqual(self.rados_inst.ioctx, ret[1])
+        self.rados_inst.open_ioctx.assert_called_with(self.rbd_pool)
 
     def test_connect_to_rados_different_pool(self):
         ret = self.driver._connect_to_rados('alt_pool')
-        self.mock_rados.Rados.connect.assert_called_once_with(
+        self.rados_inst.connect.assert_called_once_with(
                 timeout=self.rbd_connect_timeout)
-        self.assertTrue(self.mock_rados.Rados.open_ioctx.called)
-        self.assertIsInstance(ret[0], self.mock_rados.Rados)
-        self.assertEqual(self.mock_rados.Rados.ioctx, ret[1])
-        self.mock_rados.Rados.open_ioctx.assert_called_with('alt_pool')
+        self.assertTrue(self.rados_inst.open_ioctx.called)
+        self.assertEqual(self.rados_inst.ioctx, ret[1])
+        self.rados_inst.open_ioctx.assert_called_with('alt_pool')
 
     def test_connect_to_rados_error(self):
-        self.mock_rados.Rados.open_ioctx.side_effect = self.mock_rados.Error
+        self.rados_inst.open_ioctx.side_effect = self.mock_rados.Error
         self.assertRaises(self.mock_rados.Error,
                           self.driver._connect_to_rados)
-        self.mock_rados.Rados.open_ioctx.assert_called_once_with(
+        self.rados_inst.open_ioctx.assert_called_once_with(
             self.rbd_pool)
-        self.mock_rados.Rados.shutdown.assert_called_once_with()
+        self.rados_inst.shutdown.assert_called_once_with()
 
     def test_connect_to_rados_unicode_arg(self):
         self.driver._connect_to_rados(u'unicode_pool')
-        self.mock_rados.Rados.open_ioctx.assert_called_with(
+        self.rados_inst.open_ioctx.assert_called_with(
             test.MatchType(str))
 
     def test_ceph_args_none(self):
@@ -523,7 +524,7 @@ class RbdTestCase(test.NoDBTestCase):
         self.driver.destroy_volume(vol)
 
         # Make sure both params have the expected values
-        retryctx = mock_loopingcall.call_args.args[3]
+        retryctx = mock_loopingcall.call_args[0][3]
         self.assertEqual(retryctx, {'retries': 6})
         loopingcall.start.assert_called_with(interval=10)
 
@@ -653,6 +654,11 @@ class RbdTestCase(test.NoDBTestCase):
                     'used': ceph_df_json['pools'][1]['stats']['bytes_used']}
         self.assertDictEqual(expected, self.driver.get_pool_info())
 
+    @mock.patch('oslo_concurrency.processutils.execute', autospec=True,
+                side_effect=processutils.ProcessExecutionError("failed"))
+    def test_get_pool_info_execute_failed(self, mock_execute):
+        self.assertRaises(exception.StorageError, self.driver.get_pool_info)
+
     @mock.patch('oslo_concurrency.processutils.execute')
     def test_get_pool_info_not_found(self, mock_execute):
         # Make the pool something other than self.rbd_pool so it won't be found
