@@ -17,7 +17,6 @@ import re
 
 import fixtures
 from jsonschema import exceptions as jsonschema_exc
-import six
 
 from nova.api.openstack import api_version_request as api_version
 from nova.api import validation
@@ -80,7 +79,7 @@ class ValidationRegex(test.NoDBTestCase):
         # about. The algorithm works for all ranges.
         def _get_all_chars():
             for i in range(0x7F):
-                yield six.unichr(i)
+                yield chr(i)
 
         self.useFixture(fixtures.MonkeyPatch(
             'nova.api.validation.parameter_types._get_all_chars',
@@ -159,7 +158,7 @@ class APIValidationTestCase(test.NoDBTestCase):
 class FormatCheckerTestCase(test.NoDBTestCase):
 
     def _format_checker(self, format, value, error_message):
-        format_checker = validators.FormatChecker()
+        format_checker = validators._FORMAT_CHECKER
         exc = self.assertRaises(jsonschema_exc.FormatError,
                                 format_checker.check, value, format)
         self.assertIsInstance(exc.cause, exception.InvalidName)
@@ -284,7 +283,7 @@ class QueryParamsSchemaTestCase(test.NoDBTestCase):
         req.api_version_request = api_version.APIVersionRequest("2.1")
         ex = self.assertRaises(
             exception.ValidationError, self.controller.get, req)
-        self.assertIn("Query string is not UTF-8 encoded", six.text_type(ex))
+        self.assertIn("Query string is not UTF-8 encoded", str(ex))
 
     def test_strip_out_additional_properties(self):
         req = fakes.HTTPRequest.blank(
@@ -501,8 +500,12 @@ class StringLengthTestCase(APIValidationTestCase):
                          'Validation succeeded.')
 
     def test_validate_string_length_fails(self):
-        detail = ("Invalid input for field/attribute foo. Value: ."
-                  " '' is too short")
+        detail = [
+            "Invalid input for field/attribute foo. Value: . '' "
+            "is too short",  # jsonschema < 4.23.0
+            "Invalid input for field/attribute foo. Value: . '' "
+            "should be non-empty",  # jsonschema >= 4.23.0
+        ]
         self.check_validation_error(self.post, body={'foo': ''},
                                     expected_detail=detail)
 
@@ -549,9 +552,9 @@ class IntegerTestCase(APIValidationTestCase):
         self.check_validation_error(self.post, body={'foo': '0xffff'},
                                     expected_detail=detail)
 
-        detail = ("Invalid input for field/attribute foo. Value: 1.0."
-                  " 1.0 is not of type 'integer', 'string'")
-        self.check_validation_error(self.post, body={'foo': 1.0},
+        detail = ("Invalid input for field/attribute foo. Value: 1.01."
+                  " 1.01 is not of type 'integer', 'string'")
+        self.check_validation_error(self.post, body={'foo': 1.01},
                                     expected_detail=detail)
 
         detail = ("Invalid input for field/attribute foo. Value: 1.0."
@@ -644,16 +647,16 @@ class BooleanTestCase(APIValidationTestCase):
                                     expected_detail=detail)
 
 
-class HostnameTestCase(APIValidationTestCase):
+class FQDNTestCase(APIValidationTestCase):
 
     post_schema = {
         'type': 'object',
         'properties': {
-            'foo': parameter_types.hostname,
+            'foo': parameter_types.fqdn,
         },
     }
 
-    def test_validate_hostname(self):
+    def test_validate_fqdn(self):
         self.assertEqual('Validation succeeded.',
                          self.post(body={'foo': 'localhost'},
                                    req=FakeRequest()))
@@ -665,7 +668,7 @@ class HostnameTestCase(APIValidationTestCase):
         self.assertEqual('Validation succeeded.',
                          self.post(body={'foo': 'my_host'}, req=FakeRequest()))
 
-    def test_validate_hostname_fails(self):
+    def test_validate_fqdn_fails(self):
         detail = ("Invalid input for field/attribute foo. Value: True."
                   " True is not of type 'string'")
         self.check_validation_error(self.post, body={'foo': True},
@@ -682,50 +685,6 @@ class HostnameTestCase(APIValidationTestCase):
                                     expected_detail=detail)
 
 
-class HostnameIPaddressTestCase(APIValidationTestCase):
-
-    post_schema = {
-        'type': 'object',
-        'properties': {
-            'foo': parameter_types.hostname_or_ip_address,
-        },
-    }
-
-    def test_validate_hostname_or_ip_address(self):
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'localhost'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'localhost.localdomain.com'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'my-host'}, req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'my_host'}, req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': '192.168.10.100'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': '2001:db8::9abc'},
-                                   req=FakeRequest()))
-
-    def test_validate_hostname_or_ip_address_fails(self):
-        detail = ("Invalid input for field/attribute foo. Value: True."
-                  " True is not of type 'string'")
-        self.check_validation_error(self.post, body={'foo': True},
-                                    expected_detail=detail)
-
-        detail = ("Invalid input for field/attribute foo. Value: 1."
-                  " 1 is not of type 'string'")
-        self.check_validation_error(self.post, body={'foo': 1},
-                                    expected_detail=detail)
-
-        detail = ("Invalid input for field/attribute foo. Value: my$host."
-                  " 'my$host' does not match '^[a-zA-Z0-9-_.:]*$'")
-        self.check_validation_error(self.post, body={'foo': 'my$host'},
-                                    expected_detail=detail)
-
-
 class NameTestCase(APIValidationTestCase):
 
     post_schema = {
