@@ -17,10 +17,10 @@
 Tests for Block Device utility functions.
 """
 
-import mock
+from unittest import mock
+
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import units
-import six
 
 from nova import block_device
 from nova.compute import api as compute_api
@@ -630,8 +630,7 @@ class TestBlockDeviceDict(test.NoDBTestCase):
         ex = self.assertRaises(exception.InvalidBDMFormat,
                                block_device.BlockDeviceDict.from_api,
                                api_dict, False)
-        self.assertIn('Mapping image to local is not supported',
-                      six.text_type(ex))
+        self.assertIn('Mapping image to local is not supported', str(ex))
 
     def test_from_api_invalid_volume_type_to_destination_local_mapping(self):
         api_dict = {'id': 1,
@@ -643,7 +642,7 @@ class TestBlockDeviceDict(test.NoDBTestCase):
                                block_device.BlockDeviceDict.from_api,
                                api_dict, False)
         self.assertIn('Specifying a volume_type with destination_type=local '
-                      'is not supported', six.text_type(ex))
+                      'is not supported', str(ex))
 
     def test_from_api_invalid_specify_volume_type_with_source_volume_mapping(
             self):
@@ -656,46 +655,7 @@ class TestBlockDeviceDict(test.NoDBTestCase):
                                block_device.BlockDeviceDict.from_api,
                                api_dict, False)
         self.assertIn('Specifying volume type to existing volume is '
-                      'not supported', six.text_type(ex))
-
-    def test_legacy(self):
-        for legacy, new in zip(self.legacy_mapping, self.new_mapping):
-            self.assertThat(
-                legacy,
-                matchers.IsSubDictOf(new.legacy()))
-
-    def test_legacy_mapping(self):
-        got_legacy = block_device.legacy_mapping(self.new_mapping)
-
-        for legacy, expected in zip(got_legacy, self.legacy_mapping):
-            self.assertThat(expected, matchers.IsSubDictOf(legacy))
-
-    def test_legacy_source_image(self):
-        for legacy, new in zip(self.legacy_mapping_source_image,
-                               self.new_mapping_source_image):
-            if new['destination_type'] == 'volume':
-                self.assertThat(legacy, matchers.IsSubDictOf(new.legacy()))
-            else:
-                self.assertRaises(exception.InvalidBDMForLegacy, new.legacy)
-
-    def test_legacy_mapping_source_image(self):
-        got_legacy = block_device.legacy_mapping(self.new_mapping)
-
-        for legacy, expected in zip(got_legacy, self.legacy_mapping):
-            self.assertThat(expected, matchers.IsSubDictOf(legacy))
-
-    def test_legacy_mapping_from_object_list(self):
-        bdm1 = objects.BlockDeviceMapping()
-        bdm1 = objects.BlockDeviceMapping._from_db_object(
-            None, bdm1, fake_block_device.FakeDbBlockDeviceDict(
-                self.new_mapping[0]))
-        bdm2 = objects.BlockDeviceMapping()
-        bdm2 = objects.BlockDeviceMapping._from_db_object(
-            None, bdm2, fake_block_device.FakeDbBlockDeviceDict(
-                self.new_mapping[1]))
-        bdmlist = objects.BlockDeviceMappingList()
-        bdmlist.objects = [bdm1, bdm2]
-        block_device.legacy_mapping(bdmlist)
+                      'not supported', str(ex))
 
     def test_image_mapping(self):
         removed_fields = ['id', 'instance_uuid', 'connection_info',
