@@ -58,11 +58,11 @@ class HackingTestCase(test.NoDBTestCase):
 
         self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
             "from nova.virt.libvirt import utils as libvirt_utils",
-            "./nova/virt/hyperv/driver.py"))
+            "./nova/virt/zvm/driver.py"))
 
         self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
             "import nova.virt.libvirt.utils as libvirt_utils",
-            "./nova/virt/hyperv/driver.py"))
+            "./nova/virt/zvm/driver.py"))
 
         self.assertIsNone(checks.import_no_virt_driver_import_deps(
             "from nova.virt.libvirt import utils as libvirt_utils",
@@ -72,7 +72,7 @@ class HackingTestCase(test.NoDBTestCase):
         self.assertIsInstance(checks.import_no_virt_driver_config_deps(
             "CONF.import_opt('volume_drivers', "
             "'nova.virt.libvirt.driver', group='libvirt')",
-            "./nova/virt/hyperv/driver.py"), tuple)
+            "./nova/virt/zm/driver.py"), tuple)
 
         self.assertIsNone(checks.import_no_virt_driver_config_deps(
             "CONF.import_opt('volume_drivers', "
@@ -517,7 +517,7 @@ class HackingTestCase(test.NoDBTestCase):
                foobar_cmd = "foobar -get -beer"
                answer = os.popen(foobar_cmd).read()
 
-               if answer == nok":
+               if answer == "nok":
                    try:
                        os.popen(os.popen('foobar -beer -please')).read()
 
@@ -1043,3 +1043,24 @@ class HackingTestCase(test.NoDBTestCase):
                     import unittest.mock
                """
         self._assert_has_no_errors(code, checks.import_stock_mock)
+
+    def test_check_set_daemon(self):
+        code = """
+                   self.setDaemon(True)
+                   worker.setDaemon(True)
+                   self._event_thread.setDaemon(True)
+                   mythread.setDaemon(False)
+                   self.thread.setDaemon(1)
+               """
+        errors = [(x + 1, 0, 'N372') for x in range(5)]
+        self._assert_has_errors(
+            code, checks.check_set_daemon, expected_errors=errors)
+
+        code = """
+                   self.setDaemon = True
+                   worker.setDaemonFlag(True)
+                   self._event_thread.resetDaemon(True)
+                   self.set.Daemon(True)
+                   self.thread.setdaemon(True)
+               """
+        self._assert_has_no_errors(code, checks.check_set_daemon)
