@@ -18,7 +18,7 @@ from unittest import mock
 
 import fixtures
 
-from nova.db import api as db
+from nova.db.main import api as db
 from nova import test
 from nova.tests.unit import utils as test_utils
 
@@ -60,6 +60,5 @@ class TestUtilsTestCase(test.TestCase):
                 self.assertTrue(test_utils.is_ipv6_supported())
 
             with fixtures.MonkeyPatch('sys.platform', 'linux2'):
-                with fixtures.MonkeyPatch('six.moves.builtins.open',
-                                          fake_open):
+                with fixtures.MonkeyPatch('builtins.open', fake_open):
                     self.assertFalse(test_utils.is_ipv6_supported())
