@@ -44,7 +44,6 @@ import oslo_messaging as messaging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
-from oslo_utils.secretutils import md5
 from oslo_utils import strutils
 from oslo_utils import timeutils
 
@@ -791,7 +790,7 @@ def get_hash_str(base_str):
     """
     if isinstance(base_str, str):
         base_str = base_str.encode('utf-8')
-    return md5(base_str, usedforsecurity=False).hexdigest()
+    return hashlib.md5(base_str, usedforsecurity=False).hexdigest()
 
 
 def get_sha256_str(base_str):
@@ -966,7 +965,10 @@ def get_ksa_adapter(service_type, ksa_auth=None, ksa_session=None,
         min_version=min_version, max_version=max_version, raise_exc=False)
 
 
-def get_sdk_adapter(service_type, check_service=False):
+def get_sdk_adapter(
+    service_type, admin, check_service=False, conf_group=None,
+    context=None, **kwargs
+):
     """Construct an openstacksdk-brokered Adapter for a given service type.
 
     We expect to find a conf group whose name corresponds to the service_type's
@@ -975,24 +977,62 @@ def get_sdk_adapter(service_type, check_service=False):
 
     :param service_type: String name of the service type for which the Adapter
                          is to be constructed.
+    :param admin: If set to true, the service will use Nova's service user
+                  and password; otherwise, it will use the user's token.
     :param check_service: If True, we will query the endpoint to make sure the
             service is alive, raising ServiceUnavailable if it is not.
+    :param conf_group: String name of the conf group to use, otherwise the name
+            of the service_type will be used.
+    :param context: Use to get user's token, if admin is set to False.
+    :param kwargs: Additional arguments to pass to the Adapter constructor.
+                   Mainly used to pass microversion to a specific service,
+                   e.g. shared_file_system_api_version="2.82".
     :return: An openstack.proxy.Proxy object for the specified service_type.
     :raise: ConfGroupForServiceTypeNotFound If no conf group name could be
             found for the specified service_type.
     :raise: ServiceUnavailable if check_service is True and the service is down
     """
-    confgrp = _get_conf_group(service_type)
-    sess = _get_auth_and_session(confgrp)[1]
+    confgrp = conf_group or _get_conf_group(service_type)
+
     try:
-        conn = connection.Connection(
-            session=sess, oslo_conf=CONF, service_types={service_type},
-            strict_proxies=check_service)
+        if admin is False:
+            if context is None:
+                raise ValueError(
+                    "If admin is set to False then context cannot be None.")
+
+            # NOTE(gibi): this is only needed to make sure
+            # CONF.service_user.auth_url config is registered
+            ks_loading.load_auth_from_conf_options(
+                CONF, nova.conf.service_token.service_user.name)
+
+            # Create a connection using the user's token instead of nova's
+            # service user/pass.
+            conn = connection.Connection(
+                token=context.auth_token,
+                auth_type="v3token",
+                project_id=context.project_id,
+                project_domain_id=context.project_domain_id,
+                auth_url=CONF.service_user.auth_url,
+                service_types={service_type},
+                strict_proxies=check_service,
+                **kwargs,
+            )
+        else:
+            # Create a connection based on nova's service user/pass
+            sess = _get_auth_and_session(confgrp)[1]
+            conn = connection.Connection(
+                session=sess, oslo_conf=CONF, service_types={service_type},
+                strict_proxies=check_service, **kwargs)
+
     except sdk_exc.ServiceDiscoveryException as e:
         raise exception.ServiceUnavailable(
             _("The %(service_type)s service is unavailable: %(error)s") %
             {'service_type': service_type, 'error': str(e)})
-    return getattr(conn, service_type)
+    # The replace('-', '_') below is to handle service names that use
+    # hyphens and SDK attributes that use underscores.
+    # e.g. service name --> sdk attribute
+    #      'shared-file-system' --> 'shared_file_system'
+    return getattr(conn, service_type.replace('-', '_'))
 
 
 def get_endpoint(ksa_adapter):
@@ -1154,42 +1194,3 @@ def run_once(message, logger, cleanup=None):
         wrapper.reset = functools.partial(reset, wrapper)
         return wrapper
     return outer_wrapper
-
-
-class _SentinelException(Exception):
-    """This type exists to act as a placeholder and will never be raised"""
-
-
-def latch_error_on_raise(retryable=(_SentinelException,)):
-    """This is a utility decorator to ensure if a function ever raises
-    it will always raise the same exception going forward.
-
-    The only exception we know is safe to ignore is an oslo db connection
-    error as the db may be temporarily unavailable and we should allow
-    mod_wsgi to retry
-    """
-
-    def outer_wrapper(func):
-        @functools.wraps(func)
-        def wrapper(*args, **kwargs):
-            if wrapper.error:
-                raise wrapper.error
-            try:
-                return func(*args, **kwargs)
-            except retryable:
-                # reraise any retryable exception to allow them to be handled
-                # by the caller.
-                raise
-            except Exception as e:
-                wrapper.error = e
-                LOG.exception(e)
-                raise
-
-        wrapper.error = None
-
-        def reset(wrapper):
-            wrapper.error = None
-
-        wrapper.reset = functools.partial(reset, wrapper)
-        return wrapper
-    return outer_wrapper
