@@ -71,12 +71,10 @@ _POWER_STATE_MAP = {
 
 _UNPROVISION_STATES = (ironic_states.ACTIVE, ironic_states.DEPLOYFAIL,
                        ironic_states.ERROR, ironic_states.DEPLOYWAIT,
-                       ironic_states.DEPLOYING, ironic_states.DEPLOYHOLD,
-                       ironic_states.RESCUE, ironic_states.RESCUING,
-                       ironic_states.RESCUEWAIT, ironic_states.RESCUEFAIL,
-                       ironic_states.UNRESCUING, ironic_states.UNRESCUEFAIL,
-                       ironic_states.SERVICING, ironic_states.SERVICEWAIT,
-                       ironic_states.SERVICEFAIL, ironic_states.SERVICEHOLD)
+                       ironic_states.DEPLOYING, ironic_states.RESCUE,
+                       ironic_states.RESCUING, ironic_states.RESCUEWAIT,
+                       ironic_states.RESCUEFAIL, ironic_states.UNRESCUING,
+                       ironic_states.UNRESCUEFAIL)
 
 _NODE_FIELDS = ('uuid', 'power_state', 'target_power_state', 'provision_state',
                 'target_provision_state', 'last_error', 'maintenance',
@@ -166,6 +164,8 @@ class IronicDriver(virt_driver.ComputeDriver):
         "supports_address_space_passthrough": False,
         "supports_address_space_emulated": False,
         "supports_stateless_firmware": False,
+        "supports_virtio_fs": False,
+        "supports_mem_backing_file": False,
 
         # Image type support flags
         "supports_image_type_aki": False,
@@ -200,7 +200,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             # service isn't ready yet. Consumers of ironic_connection are set
             # up to handle this and raise VirtDriverNotReady as appropriate.
             self._ironic_connection = utils.get_sdk_adapter(
-                'baremetal', check_service=True)
+                'baremetal', admin=True, check_service=True)
         return self._ironic_connection
 
     def _get_node(self, node_id):
@@ -1213,20 +1213,14 @@ class IronicDriver(virt_driver.ComputeDriver):
         ):
             # something is wrong. undo what we have done
             self._cleanup_deploy(node, instance, network_info)
-            deploy_msg = ("No Error" if validate_chk['deploy'].result
-                          else validate_chk['deploy'].reason)
-            power_msg = ("No Error" if validate_chk['power'].result
-                         else validate_chk['power'].reason)
-            storage_msg = ("No Error" if validate_chk['storage'].result
-                           else validate_chk['storage'].reason)
             raise exception.ValidationError(_(
                 "Ironic node: %(id)s failed to validate. "
                 "(deploy: %(deploy)s, power: %(power)s, "
                 "storage: %(storage)s)")
                 % {'id': node.id,
-                   'deploy': deploy_msg,
-                   'power': power_msg,
-                   'storage': storage_msg})
+                   'deploy': validate_chk['deploy'],
+                   'power': validate_chk['power'],
+                   'storage': validate_chk['storage']})
 
         # Config drive
         configdrive_value = None
@@ -1370,18 +1364,13 @@ class IronicDriver(virt_driver.ComputeDriver):
             #             without raising any exceptions.
             return
 
-        if (node.provision_state in _UNPROVISION_STATES or
-            node.provision_state not in ironic_states.PROVISION_STATE_LIST):
+        if node.provision_state in _UNPROVISION_STATES:
             # NOTE(mgoddard): Ironic's node tear-down procedure includes all of
             # the things we do in _cleanup_deploy, so let's not repeat them
             # here. Doing so would also race with the node cleaning process,
             # which may acquire the node lock and prevent us from making
             # changes to the node. See
             # https://bugs.launchpad.net/nova/+bug/2019977
-            # NOTE(JayF): For maximum safety, we call unprovision also in cases
-            # where we see an unrecognized state. See
-            # https://bugs.launchpad.net/nova/+bug/2131960 for more
-            # information.
             self._unprovision(instance, node)
         else:
             self._cleanup_deploy(node, instance, network_info)
@@ -1391,7 +1380,7 @@ class IronicDriver(virt_driver.ComputeDriver):
 
     def reboot(self, context, instance, network_info, reboot_type,
                block_device_info=None, bad_volumes_callback=None,
-               accel_info=None):
+               accel_info=None, share_info=None):
         """Reboot the specified instance.
 
         NOTE: Unlike the libvirt driver, this method does not delete
@@ -1408,6 +1397,8 @@ class IronicDriver(virt_driver.ComputeDriver):
             encountered. Ignored by this driver.
         :param accel_info: List of accelerator request dicts. The exact
             data struct is doc'd in nova/virt/driver.py::spawn().
+        :param share_info: share mapping information used to mount Manila
+            shares on the compute and then on the instance using virtiofs.
         """
         LOG.debug('Reboot(type %s) called for instance',
                   reboot_type, instance=instance)
@@ -1502,7 +1493,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                  node.id, instance=instance)
 
     def power_on(self, context, instance, network_info,
-                 block_device_info=None, accel_info=None):
+                 block_device_info=None, accel_info=None, share_info=None):
         """Power on the specified instance.
 
         NOTE: Unlike the libvirt driver, this method does not delete
@@ -1516,6 +1507,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             information. Ignored by this driver.
         :param accel_info: List of accelerator requests for this instance.
                            Ignored by this driver.
+        :param share_info: instance share attached list.
         """
         LOG.debug('Power on called for instance', instance=instance)
         node = self._validate_instance_and_node(instance)
@@ -1866,7 +1858,10 @@ class IronicDriver(virt_driver.ComputeDriver):
         def _enable_console(mode):
             """Request to enable/disable node console."""
             try:
-                self.ironic_connection.set_node_console_mode(node_id, mode)
+                if mode:
+                    self.ironic_connection.enable_node_console(node_id)
+                else:
+                    self.ironic_connection.disable_node_console(node_id)
             except sdk_exc.SDKException as e:
                 LOG.error('Failed to set console mode to "%(mode)s" '
                           'for instance %(inst)s: %(reason)s',
@@ -2152,7 +2147,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             raise exception.IronicAPIVersionNotAvailable(version=version)
 
     def rescue(self, context, instance, network_info, image_meta,
-               rescue_password, block_device_info):
+               rescue_password, block_device_info, share_info):
         """Rescue the specified instance.
 
         :param nova.context.RequestContext context:
@@ -2167,6 +2162,8 @@ class IronicDriver(virt_driver.ComputeDriver):
         :param rescue_password: new root password to set for rescue.
         :param dict block_device_info:
             The block device mapping of the instance.
+        :param nova.objects.share_mapping.ShareMapingList share_info
+            optional list of share_mapping
         :raise InstanceRescueFailure if rescue fails.
         """
         LOG.debug('Rescue called for instance', instance=instance)
