@@ -261,7 +261,7 @@ class Guest(object):
                 LOG.debug(f'No interface of type: {type(cfg)} found in domain')
                 return None
             # FIXME(sean-k-mooney): we should be able to print the list of
-            # interfaces however some tests use incomplete objects that cant
+            # interfaces however some tests use incomplete objects that can't
             # be printed due to incomplete mocks or defects in the libvirt
             # fixture. Lets address this later.
             # LOG.debug(f'within interfaces: {list(interfaces)}')
@@ -295,12 +295,11 @@ class Guest(object):
             yield VCPUInfo(
                 id=vcpu[0], cpu=vcpu[3], state=vcpu[1], time=vcpu[2])
 
-    def delete_configuration(self, support_uefi=False):
+    def delete_configuration(self):
         """Undefines a domain from hypervisor."""
         try:
             flags = libvirt.VIR_DOMAIN_UNDEFINE_MANAGED_SAVE
-            if support_uefi:
-                flags |= libvirt.VIR_DOMAIN_UNDEFINE_NVRAM
+            flags |= libvirt.VIR_DOMAIN_UNDEFINE_NVRAM
             self._domain.undefineFlags(flags)
         except libvirt.libvirtError:
             LOG.debug("Error from libvirt during undefineFlags for guest "
@@ -333,8 +332,6 @@ class Guest(object):
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
 
         device_xml = conf.to_xml()
-        if isinstance(device_xml, bytes):
-            device_xml = device_xml.decode('utf-8')
 
         LOG.debug("attach device xml: %s", device_xml)
         self._domain.attachDeviceFlags(device_xml, flags=flags)
@@ -414,6 +411,12 @@ class Guest(object):
 
         return self.get_all_devices(vconfig.LibvirtConfigGuestDisk)
 
+    def get_device_by_alias(self, devalias, devtype=None,
+                            from_persistent_config=False):
+        for dev in self.get_all_devices(devtype):
+            if hasattr(dev, 'alias') and dev.alias == devalias:
+                return dev
+
     def get_all_devices(
         self,
         devtype: vconfig.LibvirtConfigGuestDevice = None,
@@ -459,8 +462,6 @@ class Guest(object):
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
 
         device_xml = conf.to_xml()
-        if isinstance(device_xml, bytes):
-            device_xml = device_xml.decode('utf-8')
 
         LOG.debug("detach device xml: %s", device_xml)
         self._domain.detachDeviceFlags(device_xml, flags=flags)
@@ -563,8 +564,6 @@ class Guest(object):
         flags |= quiesce and libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_QUIESCE or 0
 
         device_xml = conf.to_xml()
-        if isinstance(device_xml, bytes):
-            device_xml = device_xml.decode('utf-8')
 
         self._domain.snapshotCreateXML(device_xml, flags=flags)
 
