@@ -22,6 +22,7 @@ import grp
 import os
 import pwd
 import re
+import tempfile
 import typing as ty
 import uuid
 
@@ -29,6 +30,7 @@ import os_traits
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import fileutils
+from oslo_utils.imageutils import format_inspector
 
 import nova.conf
 from nova import context as nova_context
@@ -102,18 +104,37 @@ CPU_TRAITS_MAPPING = {
     'xop': os_traits.HW_CPU_X86_XOP
 }
 
+
+def make_reverse_cpu_traits_mapping() -> ty.Dict[str, str]:
+    traits_cpu_mapping = dict()
+    for k, v in CPU_TRAITS_MAPPING.items():
+        if isinstance(v, tuple):
+            for trait in v:
+                traits_cpu_mapping[trait] = k
+        else:
+            traits_cpu_mapping[v] = k
+    return traits_cpu_mapping
+
+
 # Reverse CPU_TRAITS_MAPPING
-TRAITS_CPU_MAPPING = {v: k for k, v in CPU_TRAITS_MAPPING.items()}
+TRAITS_CPU_MAPPING = make_reverse_cpu_traits_mapping()
 
 # global directory for emulated TPM
 VTPM_DIR = '/var/lib/libvirt/swtpm/'
 
 
+class EncryptionOptions(ty.TypedDict):
+    secret: str
+    format: str
+
+
 def create_image(
     path: str,
     disk_format: str,
     disk_size: ty.Optional[ty.Union[str, int]],
     backing_file: ty.Optional[str] = None,
+    encryption: ty.Optional[EncryptionOptions] = None,
+    safe: bool = False,
 ) -> None:
     """Disk image creation with qemu-img
     :param path: Desired location of the disk image
@@ -125,15 +146,52 @@ def create_image(
         If no suffix is given, it will be interpreted as bytes.
         Can be None in the case of a COW image.
     :param backing_file: (Optional) Backing file to use.
+    :param encryption: (Optional) Dict detailing various encryption attributes
+        such as the format and passphrase.
+    :param safe: If True, the image is know to be safe.
     """
-    base_cmd = [
+    cmd = [
         'env', 'LC_ALL=C', 'LANG=C', 'qemu-img', 'create', '-f', disk_format
     ]
-    cow_opts = []
 
     if backing_file:
+        # NOTE(danms): We need to perform safety checks on the base image
+        # before we inspect it for other attributes. We do this each time
+        # because additional safety checks could have been added since we
+        # downloaded the image.
+        # Note(sean-k-mooney): We only need to perform the safety check for
+        # the backing file if the image is not created by nova for swap or
+        # ephemeral disks.
+        if not CONF.workarounds.disable_deep_image_inspection and not safe:
+            inspector = images.get_image_format(backing_file)
+            try:
+                inspector.safety_check()
+            except format_inspector.SafetyCheckFailed as e:
+                LOG.warning('Base image %s failed safety check: %s',
+                            backing_file, e)
+                # NOTE(danms): This is the same exception as would be raised
+                # by qemu_img_info() if the disk format was unreadable or
+                # otherwise unsuitable.
+                raise exception.InvalidDiskInfo(
+                    reason=_('Base image failed safety check'))
+
         base_details = images.qemu_img_info(backing_file)
-        cow_opts += [
+        if base_details.file_format == 'vmdk':
+            images.check_vmdk_image('base', base_details)
+        if base_details.backing_file is not None:
+            LOG.warning('Base image %s failed safety check', backing_file)
+            raise exception.InvalidDiskInfo(
+                reason=_('Base image failed safety check'))
+        try:
+            data_file = base_details.format_specific['data']['data-file']
+        except (KeyError, TypeError, AttributeError):
+            data_file = None
+        if data_file is not None:
+            LOG.warning('Base image %s failed safety check', backing_file)
+            raise exception.InvalidDiskInfo(
+                reason=_('Base image failed safety check'))
+
+        cow_opts = [
             f'backing_file={backing_file}',
             f'backing_fmt={base_details.file_format}'
         ]
@@ -147,12 +205,60 @@ def create_image(
 
         # Format as a comma separated list
         csv_opts = ",".join(cow_opts)
-        cow_opts = ['-o', csv_opts]
+        cmd += ['-o', csv_opts]
 
-    cmd = base_cmd + cow_opts + [path]
-    if disk_size is not None:
-        cmd += [str(disk_size)]
-    processutils.execute(*cmd)
+    # Disk size can be None in the case of a COW image
+    disk_size_arg = [str(disk_size)] if disk_size is not None else []
+
+    if encryption:
+        with tempfile.NamedTemporaryFile(mode='tr+', encoding='utf-8') as f:
+            # Write out the passphrase secret to a temp file
+            f.write(encryption['secret'])
+
+            # Ensure the secret is written to disk, we can't .close() here as
+            # that removes the file when using NamedTemporaryFile
+            f.flush()
+
+            # The basic options include the secret and encryption format
+            encryption_opts = [
+                '--object', f"secret,id=sec,file={f.name}",
+                '-o', 'encrypt.key-secret=sec',
+                '-o', f"encrypt.format={encryption['format']}",
+            ]
+            # Supported luks options:
+            #  cipher-alg=<str>       - Name of cipher algorithm and key length
+            #  cipher-mode=<str>      - Name of encryption cipher mode
+            #  hash-alg=<str>         - Name of hash algorithm to use for PBKDF
+            #  iter-time=<num>        - Time to spend in PBKDF in milliseconds
+            #  ivgen-alg=<str>        - Name of IV generator algorithm
+            #  ivgen-hash-alg=<str>   - Name of IV generator hash algorithm
+            #
+            # NOTE(melwitt): Sensible defaults (that match the qemu defaults)
+            # are hardcoded at this time for simplicity and consistency when
+            # instances are migrated. Configuration of luks options could be
+            # added in a future release.
+            encryption_options = {
+                'cipher-alg': 'aes-256',
+                'cipher-mode': 'xts',
+                'hash-alg': 'sha256',
+                'iter-time': 2000,
+                'ivgen-alg': 'plain64',
+                'ivgen-hash-alg': 'sha256',
+            }
+
+            for option, value in encryption_options.items():
+                encryption_opts += [
+                    '-o',
+                    f'encrypt.{option}={value}',
+                ]
+
+            # We need to execute the command while the NamedTemporaryFile still
+            # exists
+            cmd += encryption_opts + [path] + disk_size_arg
+            processutils.execute(*cmd)
+    else:
+        cmd += [path] + disk_size_arg
+        processutils.execute(*cmd)
 
 
 def create_ploop_image(
@@ -210,8 +316,8 @@ def copy_image(
     dest: str,
     host: ty.Optional[str] = None,
     receive: bool = False,
-    on_execute: ty.Callable = None,
-    on_completion: ty.Callable = None,
+    on_execute: ty.Optional[ty.Callable] = None,
+    on_completion: ty.Optional[ty.Callable] = None,
     compression: bool = True,
 ) -> None:
     """Copy a disk image to an existing directory
@@ -583,12 +689,12 @@ def mdev_name2uuid(mdev_name: str) -> str:
     :returns: string containing the uuid
     """
     mdev_uuid = mdev_name[5:].replace('_', '-')
-    # Unconditionnally remove the PCI address from the name
+    # Unconditionally remove the PCI address from the name
     mdev_uuid = mdev_uuid[:36]
     return str(uuid.UUID(mdev_uuid))
 
 
-def mdev_uuid2name(mdev_uuid: str, parent: str = None) -> str:
+def mdev_uuid2name(mdev_uuid: str, parent: ty.Optional[str] = None) -> str:
     """Convert an mdev uuid (of the form 8-4-4-4-12) and optionally its parent
     device to a name (of the form mdev_<uuid_with_underscores>[_<pciid>]).
 
