@@ -50,6 +50,8 @@ class ZVMDriver(driver.ComputeDriver):
         "supports_address_space_passthrough": False,
         "supports_address_space_emulated": False,
         "supports_stateless_firmware": False,
+        "supports_virtio_fs": False,
+        "supports_mem_backing_file": False,
 
         # Image type support flags
         "supports_image_type_aki": False,
@@ -131,7 +133,6 @@ class ZVMDriver(driver.ComputeDriver):
                                      obj_fields.HVType.ZVM,
                                      obj_fields.VMMode.HVM)],
             'numa_topology': None,
-            'stats': {'uptime': self.get_host_uptime()}
         }
 
         LOG.debug("Getting available resource for %(host)s:%(nodename)s",
@@ -399,7 +400,7 @@ class ZVMDriver(driver.ComputeDriver):
             self._hypervisor.guest_softstop(instance.name)
 
     def power_on(self, context, instance, network_info,
-                 block_device_info=None, accel_info=None):
+                 block_device_info=None, accel_info=None, share_info=None):
         self._hypervisor.guest_start(instance.name)
 
     def pause(self, instance):
@@ -410,7 +411,7 @@ class ZVMDriver(driver.ComputeDriver):
 
     def reboot(self, context, instance, network_info, reboot_type,
                block_device_info=None, bad_volumes_callback=None,
-               accel_info=None):
+               accel_info=None, share_info=None):
 
         if reboot_type == 'SOFT':
             self._hypervisor.guest_reboot(instance.name)
